@@ -18,6 +18,7 @@
 
 
 #include	<stdio.h>
+#include	"log.h"
 #if !__MACH__
 #undef _MSL_BEGIN_EXTERN_C
 #undef _MSL_END_EXTERN_C
@@ -29,7 +30,7 @@
 void
 er_panic(char * str)
 {
-	fprintf(stderr,"%s",str);
+	log_printf(LOG_SYSTEM,LOG_LAYER_LIB,0,"%s",str);
 	DisplayAlert(str);
 #if !__MACH__
 	SIOUXSettings.asktosaveonclose = true;
