@@ -41,7 +41,7 @@ struct rlimit r;
 	}
 }
 
-unsigned int
+INTEGER64
 _get_mem_size()
 {
 static unsigned int mem_size;
