@@ -27,11 +27,13 @@
 #include	<stdlib.h>
 #include	<netdb.h>
 #include	<fcntl.h>
+#include	<signal.h>
 #include	"task.h"
 #include	"memory_debug.h"
 #include	"stream.h"
 #include	"netutils.h"
 #include	"machine/fork_lock.h"
+#include	"machine/err.h"
 
 #define CON_TIMEOUT	(4*60)
 extern int max_fid,now_open_files;
@@ -43,9 +45,11 @@ typedef struct connect_t {
 	int			len;
 	int			ret;
 	int			err;
+	int			tid;
 } CONNECT_T;
 
 int con_err;
+int con_errno;
 
 void
 do_connect(CONNECT_T * arg)
@@ -55,6 +59,12 @@ do_connect(CONNECT_T * arg)
 	arg->err = errno;
 }
 
+void
+timeout_connect(CONNECT_T * c)
+{
+	throw_signal(c->tid,SIGUSR1);
+}
+
 STREAM *
 new_connection(
 	int * cerr,
@@ -100,10 +110,8 @@ CONNECT_T c;
 retry:
 	if ( get_xltime() - con_time > CON_TIMEOUT ) {
 		con_err = 4;
-		fprintf(stderr,"%s %x %i\n",hostname,ip,port);
-		perror("Error(clinet timeout): can't connect\n");
-		fprintf(stderr,"errno = %i\n",errno);
-		goto err;
+		*cerr = ESYS_CTIMEOUT;
+		goto err2;
 	}
 	/* TCP 17 */
 	errno = 0;
@@ -148,27 +156,44 @@ con_retry:
 	c.id = id;
 	c.server = (struct sockaddr*)&server;
 	c.len = len;
+	c.tid = get_tid();
+
+	new_tick(timeout_connect,-2*CON_TIMEOUT,(int)&c);
+
 	ss_do((void(*)(void*))do_connect,&c);
+
+	del_tick_with_data(timeout_connect,&c);
+
 	if ( c.ret == -1) {
 		runlock_fork(&ft);
 		if ( ft.err ) {
 			close(id);
 			goto retry;
 		}
-		if ( c.err == EINTR )
+		if ( c.err != ECONNREFUSED ) {
+			con_errno = c.err;
+			if ( get_xltime() - con_time > CON_TIMEOUT ) {
+				con_err = 4;
+				*cerr = ESYS_CTIMEOUT;
+				close(id);
+				goto err2;
+			}
+			sleep_sec(1);
 			goto con_retry;
+		}
 		if ( c.err == EISCONN )
 			goto ok;
-		fprintf(stderr,"%s %x %i\n",hostname,ip,port);
-		perror("Error(clinet): can't connect\n");
-		fprintf(stderr,"errno = %i\n",c.err);
-		con_err = 4;
+		con_err = 5;
 		close(id);
 	err:
 		if ( func )
 			(*func)(1,work);
 		*cerr = c.err;
 		return 0;
+	err2:
+		if ( func )
+			(*func)(1,work);
+		return 0;
 	}
 	runlock_fork(&ft);
 	if ( ft.err ) {
