@@ -197,7 +197,12 @@ char * _f;
 		access_mode = GENERIC_WRITE|GENERIC_READ;
 	
 	if( (mode & O_CREAT) || (mode & O_RDWR) || (mode & O_WRONLY)){
-		create_disposition = CREATE_ALWAYS;
+		if(mode & O_APPEND){
+			create_disposition = OPEN_ALWAYS;
+		}
+		else{
+			create_disposition = CREATE_ALWAYS;
+		}
 	}
 	else if(mode & O_TRUNC){
 		create_disposition = TRUNCATE_EXISTING;
@@ -222,6 +227,12 @@ char * _f;
 		d_f_ree(ret);
 		return 0;
 	}
+
+	if(mode & O_APPEND){
+		SetFilePointer(ret->file.hfile, 0, 0, FILE_END);
+	}
+
+
 	lock_task(stream_lock);
 	_s_open(ret,mode);
 	unlock_task(stream_lock,"s_open_file");
