@@ -16,8 +16,9 @@
 **********************************************************************/
 
 #include <windows.h>
+#include "utils.h"
 
-unsigned int
+INTEGER64
 get_mem_size(void)
 {
 	MEMORYSTATUS ms;
