@@ -25,6 +25,8 @@
 #include	"pri_level.h"
 extern MS_INFO *order_ms;
 
+SEM bufdc_lock;
+
 #define min(a, b)  (((a) < (b)) ? (a) : (b)) 
 
 static VDISPLAY *vd;
@@ -41,6 +43,7 @@ extern void _button(VWINDOW * win,int subtype,int t,int x,int y);
 void new_v_loop(VDISPLAY * d)
 {
 	MSG msg;
+
 	SetWindowLong(NULL, GWL_USERDATA, (int)d);
 	while( GetMessage(&msg, NULL, 0, 0) ) 
 	{
@@ -102,8 +105,7 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 	switch( message ) 
 	{
 		case WM_PAINT:
-
-			hdc = BeginPaint(hWnd, &ps);
+			
 			if(IsFirstTime)
 			{
 				win = _get_vwin(vd,hWnd);
@@ -111,6 +113,8 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				IsFirstTime = 0;
 			}
 			
+			lock_task(bufdc_lock);
+			hdc = BeginPaint(hWnd, &ps);
 			BitBlt(hdc, 
 				ps.rcPaint.left, 
 				ps.rcPaint.top,
@@ -120,8 +124,9 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 				ps.rcPaint.left,
 				ps.rcPaint.top, 
 				SRCCOPY);
-			
 			EndPaint( hWnd, &ps );
+			unlock_task(bufdc_lock,"bufdc_lock");
+
 			{
 				if(window_update_flag){
 					wakeup_task(&window_update_flag);
@@ -165,10 +170,11 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 			xPos = LOWORD(lParam);
 			yPos = HIWORD(lParam);  
 			win = _get_vwin(vd,hWnd);
+
 			if (win) 
 			{
 				VOBJECT *obj;
-				hdc = GetDC(hWnd);
+				/* hdc = GetDC(hWnd); */
 				_button(win,VES_BUTTON_MOTION,1,xPos,yPos);
 				GetClientRect(hWnd,&rt);
 				for ( obj = win->obj_list ; obj ; obj = obj->header.next )
@@ -176,7 +182,7 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 					if(obj->header.type == 2)
 						(*obj->header.handler)(obj,VE_REDRAW,rt.left,rt.top,rt.right-rt.left,rt.bottom-rt.top);
 				}
-				ReleaseDC(hWnd,hdc);
+				/* ReleaseDC(hWnd,hdc); */
 				InvalidateRect(hWnd, NULL, FALSE);
 			}
 
@@ -184,8 +190,11 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 		case WM_DESTROY:
 			{
+				lock_task(bufdc_lock);
 				DeleteObject(SelectObject(ghBufferDC,ghBufferImageOld));
 				DeleteDC(ghBufferDC);
+				unlock_task(bufdc_lock, "bufdc");
+				close_lock(bufdc_lock);
 
 				PostQuitMessage( 0 );
 			}
@@ -193,6 +202,8 @@ LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
 
 		case WM_CREATE:
 			{
+				bufdc_lock = new_lock(0);
+
 				hdc = GetDC(hWnd);
 				ghBufferDC = CreateCompatibleDC(hdc);
 				ghBufferImage = CreateCompatibleBitmap(hdc,GetSystemMetrics(SM_CXSCREEN),GetSystemMetrics(SM_CYSCREEN));
