@@ -20,56 +20,53 @@
 #include	"memory_debug.h"
 #include	"machine/msequence.h"
 #include	"v.h"
+#include	"task.h"
+
+
+extern SEM bufdc_lock;
+
 
 int _v_redraw_text(VTEXT * vt)
 {
 	HWND hwnd=vt->_h.win->w;
-	HDC hdc=vt->_h.win->gc;
 	BOOL bPaint=TRUE;
-	HPEN oldpen;
+	HFONT oldfont;
 	RECT rect;
 	SIZE size;
-	if (hdc==NULL)
-	{
-		bPaint=FALSE;
-		hdc=GetDC(vt->_h.win->w);
-	}
-	{
-		oldpen=SelectObject(ghBufferDC,vt->id);
-		SetTextColor(ghBufferDC,GB2MCOLOR_D(vt->_h.win->d,vt->color));
+	COLORREF col;
+	HDC hdc=NULL;
+	unsigned long mcolor;
 
-		if ( vt->background & C_NULL ) {
-			SetBkMode(ghBufferDC,TRANSPARENT);
-		}
-		else
-		{
-			SetBkColor(ghBufferDC,GB2MCOLOR_D(vt->_h.win->d,vt->background));
-			SetBkMode(ghBufferDC,OPAQUE);
-		}
-		rect.left = vt->st_x;
-		rect.top = vt->st_y;
-		GetTextExtentPoint32(ghBufferDC,n_string(std_cm,vt->data),vt->length,&size);
-		rect.right = vt->st_x + size.cx;
-		rect.bottom = vt->st_y + size.cy;
+	lock_task(bufdc_lock);
+	hdc=GetDC(vt->_h.win->w);
+	oldfont=SelectObject(ghBufferDC,vt->id);
+	
+	mcolor = GB2MCOLOR_D(vt->_h.win->d,vt->color);
+	col = RGBQUADToCOLORREF(*(RGBQUAD*)&mcolor);
+	SetTextColor(ghBufferDC, col); 
 
-		DrawText(ghBufferDC,n_string(std_cm,vt->data),vt->length, &rect, DT_TOP|DT_LEFT|DT_NOPREFIX);
-/*
-		if(gdwThreadID != GetCurrentThreadId())
-		{
-			PostThreadMessage(gdwThreadID, MYMSG_INVRECT, 0, 0);
-		}
-		else
-		{
-			InvalidateRect(ghWnd, &rect, FALSE);
-		}
-		*/
-
-		SelectObject(hdc,oldpen);
+	if ( vt->background & C_NULL ) {
+		SetBkMode(ghBufferDC,TRANSPARENT);
 	}
-	if (bPaint==FALSE)
+	else
 	{
-		ReleaseDC(vt->_h.win->w,hdc);
+		mcolor = GB2MCOLOR_D(vt->_h.win->d,vt->background);
+		col = RGBQUADToCOLORREF(*(RGBQUAD*)&mcolor);
+		SetBkColor(ghBufferDC,col);
+		SetBkMode(ghBufferDC,OPAQUE);
 	}
+	rect.left = vt->st_x;
+	rect.top = vt->st_y;
+	GetTextExtentPoint32(ghBufferDC,n_string(std_cm,vt->data),vt->length,&size);
+	rect.right = vt->st_x + size.cx;
+	rect.bottom = vt->st_y + size.cy;
+
+	DrawText(ghBufferDC,n_string(std_cm,vt->data),vt->length, &rect, DT_TOP|DT_LEFT|DT_NOPREFIX);
+
+	SelectObject(ghBufferDC,oldfont);
+	ReleaseDC(vt->_h.win->w,hdc);
+	unlock_task(bufdc_lock,"bufdc");
+
 	return 0;
 }
 void v_redraw_text(VTEXT * vt)
@@ -113,9 +110,7 @@ int _v_create_text(V_CREATE_TEXT_S *v)
 		DEFAULT_QUALITY, VARIABLE_PITCH|FF_ROMAN,"System");
 
 	obj = d_alloc(sizeof(VTEXT));
-//	obj->vtext.font_name = d_alloc(strlen(win->d->ascii_font.fontname)+1);
 	obj->vtext.font_name = 0;
-//	strcpy(obj->vtext.font_name,win->d->ascii_font.fontname);
 	obj->vtext.id = fid;
 	obj->vtext.st_x = v->st_x;
 	obj->vtext.st_y = v->st_y;
@@ -181,16 +176,6 @@ int _v_set_text(V_SET_TEXT_S *v)
 	L_CHAR *data = v->data;
 	int length = v->length;
 
-	//obj->vtext.color = 0xffffff & color;
-	/*
-	if ( color < 0x1000000 ) {
-		obj->vtext.color = color;
-	}
-	
-	if ( background < 0x2000000 ) {
-		obj->vtext.background = background;
-	}
-	*/
 	if ( !(color & C_NULL) ) {
 		obj->vtext.color = color;
 	}
@@ -204,11 +189,7 @@ int _v_set_text(V_SET_TEXT_S *v)
 		err->err2 = 0;
 		return 0;
 	}
-	/*
-	if (obj->header.win->d->ansi_font.fontname) {
-		//tHgǂݍ
-	}
-	*/
+	
 	if ( data ) {
 		d_f_ree(obj->vtext.data);
 		obj->vtext.data = d_alloc(sizeof(L_CHAR)*(length+1));
@@ -245,15 +226,21 @@ typedef struct v_text_minrect_s{
 
 int _v_text_minrect(V_TEXT_MINRECT_S *v)
 {
+	HDC hDC;
+	HFONT old;
 	VRECT * r = v->r;
 	VTEXT *vt=(VTEXT *)(&(v->obj->vtext));
 	RECT rect={vt->st_x,vt->st_y,vt->st_x+10,vt->st_y+10};
 	HWND hwnd=v->obj->header.win->w;
-	HDC hDC=GetDC(hwnd);
-	HFONT old=SelectObject(hDC,vt->id);
+	
+	lock_task(bufdc_lock);
+	hDC=GetDC(hwnd);
+	old=SelectObject(hDC,vt->id);
 	DrawText(ghBufferDC, n_string(std_cm, vt->data),vt->length, &rect, DT_TOP|DT_LEFT|DT_NOPREFIX|DT_CALCRECT);
 	SelectObject(hDC,old);
 	ReleaseDC(hwnd,hDC);
+	unlock_task(bufdc_lock,"bufdc");
+
 	r->tl.x=rect.left;
 	r->tl.y=rect.top;
 	r->br.x=rect.right;
