@@ -860,7 +860,7 @@ L_CHAR * filename;
 	if ( mpi->alist_fetch == 0 ) {
 		unlock_task(mpi_lock,"get_mpi_around_all");
 
-		filename = ll_copy_str(get_url_str(&mpi->u));
+		filename = ll_copy_str(get_url_str2(&mpi->u));
 		gc_push(0,0,"make_around_list");
 		chi = filtering_crd(get_crd_chi(filename));
 		par = filtering_crd(get_crd_par(filename));
@@ -889,7 +889,7 @@ L_CHAR nul[2] = {(L_CHAR)'*',0};
 		url_str = nul;
 	if ( cmp_site(up->server,0,up->port,
 			0,local_ip,p) ) {
-		log_printf(LOG_WARNING,
+		log_printf(LOG_WARNING,LOG_LAYER_GB,0,
 			"(%s)get_mpi ses=0 REMOTE_CK %ls(%ls)\n",
 			msg,get_url_str2(up),url_str);
 		gc_pop(0,0);
@@ -1036,7 +1036,8 @@ no_fit:
 	if ( cmp_site(u.server,0,u.port,
 			0,local_ip,p) && ses == 0 ) {
 	  	unlock_task(mpi_lock,"get_mpi1");
-		log_printf(LOG_WARNING,"get_mpi ses=0 REMOTE %ls(%ls)\n",
+		log_printf(LOG_WARNING,LOG_LAYER_GB,0,
+			"get_mpi ses=0 REMOTE %ls(%ls)\n",
 			get_url_str2(&u),url_str);
 		free_url(&u);
 		gc_pop(0,0);
@@ -1109,8 +1110,8 @@ set_cpu_msg(730200);
 		m->sts = sts;
 set_cpu_msg(730201);
 /*
-log_print_sexp(LOG_DEBUG,"get_mpt",ret,0);
-log_printf(LOG_DEBUG,"\n");
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,"get_mpt",ret,0);
+log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"\n");
 */
 		m->mpt = get_mpt(&reg,&cid_flag,ret);
 set_cpu_msg(730202);
