@@ -154,7 +154,7 @@ int ret;
 	if ( p == 0 )
 		return -1;
 	ret = -1;
-	h = d_alloc(sizeof(*h),1);
+	h = d_alloc(sizeof(*h));
 	h->h.fofs = 0;
 	h->d.h.size = sizeof(h->d);
 	h->d.h.type = PNT_PMD_HEADER;
@@ -259,7 +259,7 @@ retry:
 		w->path,mode,
 		0644,PF_USEFREELIST,FT_ADDRESS_LUMP,0,0,0);
 	if ( w->p == 0 ) {
-		log_printf(LOG_SYSTEM,"remake lump (%s) because open error",
+		log_printf(LOG_SYSTEM,LOG_LAYER_GB,0,"remake lump (%s) because open error",
 			n_string(std_cm,db));
 	remake:
 		if ( unit_path == 0 ) {
@@ -278,7 +278,7 @@ retry:
 		rh = get_file_record(&fofs,w->p,PNT_PMD_HEADER);
 		if ( rh == 0 ) {
 			close_filespace(w->p);
-			log_printf(LOG_SYSTEM,
+			log_printf(LOG_SYSTEM,LOG_LAYER_GB,0,
 				"remake lump (%s) because invalid format",
 				n_string(std_cm,db));
 			goto remake;
@@ -297,7 +297,7 @@ retry:
 			w->path = 0;
 			free_pmd_work(w);
 			w->path = pp;
-			log_printf(LOG_SYSTEM,
+			log_printf(LOG_SYSTEM,LOG_LAYER_GB,0,
 	"remake lump (%s) because incomplete write/invalid version/invalid unit",
 				n_string(std_cm,db));
 			goto remake;
@@ -362,13 +362,13 @@ ACC_PN_PMD_MD * md;
 			free_bib_list(ret);
 			return 0;
 		}
-		bl = d_alloc(sizeof(*bl),12);
-		bl->namespace = ll_copy_str(md->d.str,1425);
+		bl = d_alloc(sizeof(*bl));
+		bl->namespace = ll_copy_str(md->d.str);
 		bl->qualifier 
-		  = ll_copy_str(&md->d.str[md->d.qualifier],1424);
+		  = ll_copy_str(&md->d.str[md->d.qualifier]);
 		bl->inherit = md->d.inherit;
-		bl->type = ll_copy_str(&md->d.str[md->d.type],1423);
-		bl->data = ll_copy_str(&md->d.str[md->d.data],1422);
+		bl->type = ll_copy_str(&md->d.str[md->d.type]);
+		bl->data = ll_copy_str(&md->d.str[md->d.data]);
 		ret = insert_bib_list(ret,bl);
 		fofs = md->d.next_fofs;
 		d_f_ree(md);
@@ -391,7 +391,7 @@ PMD_TEMP * ret;
 		d_f_ree(n);
 		return 0;
 	}
-	ret = d_alloc(sizeof(*ret),1);
+	ret = d_alloc(sizeof(*ret));
 	ret->fofs = n->h.fofs;
 	ret->minrect = n->d.minrect;
 	ret->resolution = n->d.resolution;
@@ -400,8 +400,8 @@ PMD_TEMP * ret;
 	ret->timeout = n->d.timeout;
 	ret->md_fofs = n->d.md_fofs;
 	ret->category = n->d.category;
-	ret->crd = ll_copy_str(n->d.str,1421);
-	ret->target = ll_copy_str(&n->d.str[n->d.target],1420);
+	ret->crd = ll_copy_str(n->d.str);
+	ret->target = ll_copy_str(&n->d.str[n->d.target]);
 	ret->md_list = get_bib_list_db(w,n->d.md_fofs);
 	d_f_ree(n);
 	return ret;
@@ -484,7 +484,7 @@ PMD_TEMP * ret;
 
 	if ( r->h.bib == 0 )
 		return 0; 
-	ret = d_alloc(sizeof(*ret),23);
+	ret = d_alloc(sizeof(*ret));
 	ret->fofs = 0;
 	ret->minrect = *minrect;
 	ret->resolution = resolution;
@@ -499,8 +499,8 @@ PMD_TEMP * ret;
 	ret->timeout = 0;
 	ret->md_fofs = 0;
 	ret->category = r->h.type;
-	ret->crd = ll_copy_str(crd,1419);
-	ret->target = ll_copy_str(get_url_str2(&r->h.entry),1418);
+	ret->crd = ll_copy_str(crd);
+	ret->target = ll_copy_str(get_url_str2(&r->h.entry));
 	ret->md_list = copy_bib_list(r->h.bib);
 	return ret;
 }
@@ -547,8 +547,7 @@ int fofs;
 	type_len = l_strlen(b->type);
 	data_len = l_strlen(b->data);
 	md = d_alloc(PN_PMD_MD_S(q_len,ns_len,type_len,data_len)
-			+ sizeof(ACC_HEADER),
-			13);
+			+ sizeof(ACC_HEADER));
 	md->d.h.type = PNT_PMD_MD;
 	md->d.h.size = PN_PMD_MD_S(q_len,ns_len,type_len,data_len);
 	md->d.next_fofs = alloc_bib_list(w,b->next);
@@ -612,8 +611,7 @@ unsigned int fofs;
 	crd_len = l_strlen(t->crd);
 	target_len = l_strlen(t->target);
 	n = d_alloc(PN_PMD_NODE_S(crd_len,target_len)
-		+ sizeof(ACC_HEADER)
-		,11);
+		+ sizeof(ACC_HEADER));
 
 	switch ( type ) {
 	case WRITE_TEMP_ALLOC:
@@ -669,7 +667,7 @@ FAVT_ROOT * r;
 
 	crd_len = l_strlen(crd);
 	obj_len = l_strlen(obj);
-	ix = d_alloc(STRING_IX_LENGTH(crd_len,obj_len),123);
+	ix = d_alloc(STRING_IX_LENGTH(crd_len,obj_len));
 	ix->fofs = 0;
 	l_strcpy(ix->d,crd);
 	l_strcpy(&ix->d[crd_len+1],obj);
@@ -697,7 +695,7 @@ FAVT_ROOT * r;
 
 	crd_len = l_strlen(crd);
 	obj_len = l_strlen(obj);
-	ix = d_alloc(STRING_IX_LENGTH(crd_len,obj_len),123);
+	ix = d_alloc(STRING_IX_LENGTH(crd_len,obj_len));
 	ix->fofs = fofs;
 	l_strcpy(ix->d,crd);
 	l_strcpy(&ix->d[crd_len+1],obj);
@@ -732,7 +730,7 @@ FAVT_ROOT * r;
 
 	crd_len = l_strlen(crd);
 	obj_len = l_strlen(obj);
-	ix = d_alloc(STRING_IX_LENGTH(crd_len,obj_len),123);
+	ix = d_alloc(STRING_IX_LENGTH(crd_len,obj_len));
 	ix->fofs = 0;
 	l_strcpy(ix->d,crd);
 	l_strcpy(&ix->d[crd_len+1],obj);
@@ -764,7 +762,7 @@ ACC_PN_PMD_OFSLIST * ol, * ol2;
 		w->err = -1;
 		return -1;
 	}
-	ix = d_alloc(sizeof(int)+d_len,1);
+	ix = d_alloc(sizeof(int)+d_len);
 	*ix = fofs;
 	memcpy(&ix[1],d,d_len);
 	n = favt_search(r,root_node(r),ix,cmp_func);
@@ -795,7 +793,7 @@ ACC_PN_PMD_OFSLIST * ol, * ol2;
 		}
 		else {
 			ol = d_alloc(PN_PMD_OFSLIST_S(2)
-				+ sizeof(ACC_HEADER),111);
+				+ sizeof(ACC_HEADER));
 			ol->d.h.type = PNT_PMD_OFSLIST;
 			ol->d.h.size = PN_PMD_OFSLIST_S(2);
 			ol->d.fofs[0] = *ixp;
@@ -835,7 +833,7 @@ int ret;
 		w->err = -1;
 		return -1;
 	}
-	ix = d_alloc(sizeof(int)+d_len,1);
+	ix = d_alloc(sizeof(int)+d_len);
 	*ix = fofs;
 	memcpy(&ix[1],d,d_len);
 	n = favt_search(r,root_node(r),ix,cmp_func);
@@ -910,7 +908,7 @@ typedef struct maxmin_work {
 int
 maxmin_func(FAVT_NODE * a,MAXMIN_WORK * w)
 {
-	w->data = d_alloc(a->data_len,12);
+	w->data = d_alloc(a->data_len);
 	memcpy(w->data,a->data,a->data_len);
 	w->len = a->data_len;
 	return 1;
@@ -1247,7 +1245,7 @@ PMD_TEMP_LIST * ret;
 		free_temp(t);
 		return tl;
 	}
-	ret = d_alloc(sizeof(*ret),12);
+	ret = d_alloc(sizeof(*ret));
 	ret->next = tl;
 	ret->temp = t;
 	return ret;
@@ -1352,7 +1350,7 @@ PMD_TEMP_LIST * ret, * tl;
 	free_bound_list(b);
 
 	for ( tl = ret ; tl ; tl = tl->next ) {
-log_printf(LOG_SYSTEM,
+log_printf(LOG_SYSTEM,LOG_LAYER_GB,0,
 "DELETE PMD %i %ls %ls\n",get_xltime() - tl->temp->timeout,
 tl->temp->crd,
 tl->temp->target);
