@@ -371,8 +371,10 @@ int len,_len;
 		return 0;
 	}
 	change_endian_rt_cache_data_to_host(&d->d);
-	if ( routing_cache.err == RE_DESTROY )
+	if ( routing_cache.err == RE_DESTROY ) {
+		d_f_ree(d);
 		return 0;
+	}
 	ret = d_alloc(sizeof(*ret));
 	memset(ret,0,sizeof(*ret));
 	ret->sts = RS_COPY;
@@ -395,6 +397,7 @@ int len,_len;
 		ptr += _len+1;
 		len -= (_len+1) * sizeof(L_CHAR);
 	}
+	d_f_ree(d);
 	return ret;
 }
 
@@ -412,8 +415,10 @@ RT_CACHE_LIST * n;
 		return 0;
 	}
 	change_endian_rt_cache_list_to_host(&lst->d);
-	if ( routing_cache.err == RE_DESTROY )
+	if ( routing_cache.err == RE_DESTROY ){
+		d_f_ree(lst);
 		return 0;
+	}
 	len = lst->d.h.size - sizeof(lst->d);
 	ptr = (PN_RT_CACHE_ELEMENT*)((&lst->d)+1);
 	for ( i = 0 ; i < len/sizeof(PN_RT_CACHE_ELEMENT) ; i ++ , ptr ++ ) {
@@ -710,6 +715,7 @@ STRING_INDEX * ix;
 		ix = n->data;
 		ix->fofs = new_ixd;
 		n->h.flags |= FAF_DIRTY;
+		free_rt_cache_list(lst1);
 	}
 	else {
 		ix = n->data;
@@ -865,7 +871,8 @@ STRING_INDEX * ix;
 	ix = n->data;
 	ix->fofs = new_ixd;
 	n->h.flags |= FAF_DIRTY;
-	
+	free_rt_cache_list(lst1);
+
 }
 
 
