@@ -55,12 +55,8 @@ void gc_gb_sexp();
 	d_f_ree(f);
 
 if ( get_type(ret) == XLT_ERROR ){
-fflush(stdout);
-ss_printf("loadmeta(%i)(%s) %ls = ",get_tid(),mode,get_url_str2(&u));
-print_sexp(s_stdout,ret,0);
-ss_printf("\n");
-ss_printf("\n");
-log_print_sexp(LOG_DEBUG,"loadmeta=",ret,0);
+log_printf(LOG_ERROR,LOG_LAYER_GB,0,"loadmeta(%i)(%s) %ls = ",get_tid(),mode,get_url_str2(&u));
+log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,"loadmeta",ret,0);
 }
 	gc_pop(ret,gc_gb_sexp);
 	return ret;
