@@ -695,16 +695,14 @@ int limit;
 	else {
 	err:
 		if ( w->que_next == 0 ) {
-ss_printf("+++++++++++++++++++++ ");
-print_sexp(s_stdout,ret,0);
-ss_printf("\n");
+log_print_sexp(LOG_WARNING,LOG_LAYER_GB,0,"resource loading error",ret,0);
 			if ( extend_flag )
 				res_end_op_extend(w,LRWE_ERROR_GET1,
 					w->result,0);
 			else	res_end_op(w,LRWE_ERROR_GET1,0);
 		}
 		else {
-ss_printf("+++++++++++++++++++++ ER\n");
+log_printf(LOG_WARNING,LOG_LAYER_GB,0,"+++++++++++++++++++++ ER\n");
 			wp = w;
 			for ( ; wp ; ) {
 				wp2 = wp->que_next;
@@ -773,15 +771,24 @@ COORDINATE_UNIT * cup;
 				l_string(std_cm,"meta"),0),
 			l_string(std_cm,"category"),0);
 		if ( cat == 0 ) {
+			log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,
+				"meta_result = LRWE_ERROR_FORMAT1",
+				wp->meta_result,0);
 			res_end_op(wp,LRWE_ERROR_FORMAT1,0);
 			continue;
 		}
 		cat = get_el(cat,1);
 		if ( cat == 0 ) {
+			log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,
+				"meta_result = LRWE_ERROR_FORMAT2",
+				wp->meta_result,0);
 			res_end_op(wp,LRWE_ERROR_FORMAT2,0);
 			continue;
 		}
 		if ( get_type(cat) != XLT_STRING ) {
+			log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,
+				"meta_result = LRWE_ERROR_FORMAT3",
+				wp->meta_result,0);
 			res_end_op(wp,LRWE_ERROR_FORMAT3,0);
 			continue;
 		}
@@ -847,8 +854,11 @@ COORDINATE_UNIT * cup;
 		rr = search_resource_by_entry(&wp->url);
 		if ( rr ) {
 			lock_task(res_lock);
-			if ( rr->h.initial_status == RS_ERROR )
+			if ( rr->h.initial_status == RS_ERROR ){
+				log_printf(LOG_ERROR,LOG_LAYER_GB,0,
+						"LRWE_ERROR_HEADER_INIT");
 				_res_end_op(wp,LRWE_ERROR_HEADER_INIT,rr);
+			}
 			else {
 				wp->result = rr;
 				_res_end_op(wp,LRWE_OK,0);
@@ -875,9 +885,13 @@ COORDINATE_UNIT * cup;
 
 		ret = set_resource_header(rr,meta);
 		if ( get_type(ret) == XLT_ERROR ) {
-ss_printf("META LOADING %x ",wp->meta_result);
-print_sexp(s_stdout,wp->meta_result,0);
-ss_printf("\n");
+			log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,
+				"LRWE_ERROR_HEADER_INIT-meta_result",
+				wp->meta_result,0);
+			log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,
+				"LRWE_ERROR_HEADER_INIT-error",
+				ret,0);
+
 			set_init_status_error(rr,ret);
 			res_end_op(wp,LRWE_ERROR_HEADER_INIT,rr);
 			goto err_end;
@@ -1501,6 +1515,8 @@ int
 get_que_pri(URL * u)
 {
 int len;
+	if ( u->resource == 0 )
+		return Q_PRI_OTHER;
 	len = l_strlen(u->resource);
 	if ( l_strcmp(&u->resource[len-4],
 			l_string(std_cm,".crd")) == 0 )
@@ -1655,12 +1671,14 @@ RESOURCE *
 get_lrw(LOAD_RESOURCE_WORK * w,URL * u)
 {
 LOAD_RESOURCE_WORK * wp;
+L_CHAR * wp_str,*u_str;
 
 	wp = w;
 	for ( ; wp ; wp = wp->req_next ){
 
 		if ( wp->err != LRWE_OK )
 			continue;
+/*
 		if ( 	l_strcmp(wp->url.proto,u->proto) == 0 &&
 			l_strcmp(wp->url.server,u->server) == 0 &&
 			wp->url.port == u->port &&
@@ -1669,6 +1687,15 @@ LOAD_RESOURCE_WORK * wp;
 
 			return wp->result;
 		}
+*/
+		wp_str = get_url_str2(&wp->url);
+		u_str = get_url_str2(u);
+		if ( wp_str == 0 )
+			continue;
+		if ( u_str == 0 )
+			break;
+		if (	l_strcmp(wp_str,u_str) == 0 )
+			return wp->result;
 	}
 	return 0;
 }
@@ -1848,7 +1875,7 @@ extern RESOURCE ** res_no_hash_table;
 RESOURCE_TYPE_LIST * rt;
 int cnt,limit;
 
-printf("RESORCE CACHE INVOKE\n");
+
 	cnt = 0;
 	limit = 1;
 	for ( i = 0 ; i < RES_HASH_SIZE ; ) {
