@@ -489,6 +489,37 @@ MP_PATH * ret;
 	return 0;
 }
 
+void
+dump_route(MP_CACHE * cc)
+{
+MP_ROUTE * r;
+MP_PATH * p, * last;
+	log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"DUMP_ROUTE\n");
+	log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"::indirect=%x\n",&indirect);
+	log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"::direct=%x\n",&direct);
+	log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"::THIS=%x\n",cc);
+	for ( r = cc->root->next ; r != cc->root ; r = r->next ) {
+		log_printf(LOG_DEBUG,LOG_LAYER_GB,0,
+			"ENTRY sts=%i flags=%x lock=%i lwait=%i modify=%i update=%i\n",
+				r->sts,
+				r->flags,
+				r->lock,
+				r->lwait,
+				r->modify,
+				r->update);
+		last = 0;
+		for ( p = r->head ; p ; last = p , p = p->next ) {
+			log_printf(LOG_DEBUG,LOG_LAYER_GB,0,
+				"::path=%ls\n",p->url);
+		}
+		if ( last != r->tail )
+			log_printf(LOG_DEBUG,LOG_LAYER_GB,0,
+				"::END %ls\n",r->tail->url);
+	}
+	log_printf(LOG_DEBUG,LOG_LAYER_GB,0,
+		"DUMP_ROUTE END\n");
+}
+
 
 MP_ROUTE *
 _new_route(MP_CACHE * cc)
@@ -505,10 +536,13 @@ MP_ROUTE * r;
 	else {
 		r = cc->root->prev;
 		for ( ; r != cc->root && r->lock ; r = r->prev );
-		if ( r == cc->root )
+		if ( r == cc->root ) {
+			dump_route(cc);
 			er_panic("_new_route");
+		}
 		delete_rring(r);
 		free_path(r);
+		memset(r,0,sizeof(*r));
 		insert_rring(cc,r);
 		r->flags = 0;
 		r->sts = 0;
@@ -986,11 +1020,8 @@ int cmp;
 	switch ( cmp ) {
 	case -1:
 		*org = _get_in_file(start,target);
-		if ( *org == 0 ) {
-			ret = 0;
-			break;
-		}
-		(*org)->lock ++;
+		if ( *org )
+			(*org)->lock ++;
 		ret = _search_route(&direct,start,target);
 		if ( ret == 0 )
 			ret = 0;
@@ -1001,11 +1032,8 @@ int cmp;
 		break;
 	case 1:
 		*org = _get_in_file(target,start);
-		if ( *org == 0 ) {
-			ret = 0;
-			break;
-		}
-		(*org)->lock ++;
+		if ( *org )
+			(*org)->lock ++;
 		ret = _search_route(&direct,target,start);
 		if ( ret == 0 )
 			ret = 0;
@@ -1101,6 +1129,7 @@ URL u;
 RESOURCE * r;
 	get_url2(&u,url);
 	r = search_resource_by_entry(&u);
+	free_url(&u);
 	if ( r == 0 )
 		return 0;
 	if ( r->h.type != RT_COORDINATE )
@@ -1233,6 +1262,10 @@ MP_ROUTE * r1, * r2;
 					_flush_route(n->route);
 				free_mp_route_t(n);
 			}
+			if ( r2 == 0 ) {
+				unlock_task(route_lock,"rr_task1");
+				continue;
+			}
 			now = get_xltime();
 			diff = r2->update - r2->modify;
 			if ( diff < MIN_CACHE_EXPIRE )
@@ -1251,6 +1284,7 @@ MP_ROUTE * r1, * r2;
 				n->handle2_target = n->target;
 			}
 			else {
+				_flush_route(r2);
 				/* cache is usable */
 				unlock_task(route_lock,"rr_task1");
 				continue;
