@@ -160,10 +160,12 @@ int g_len,i;
 			m = R_NEXT(MAP*,&m->h) ) {
 		if ( bw->type == BWT_ROUTE ) {
 		int len;
-			len = l_strlen(m->src_url.resource);
-			if ( l_strcmp(&m->src_url.resource[len-4],
-					l_string(std_cm,".crd")) )
-				continue;
+			if ( m->src_url.resource ) {
+				len = l_strlen(m->src_url.resource);
+				if ( l_strcmp(&m->src_url.resource[len-4],
+						l_string(std_cm,".crd")) )
+					continue;
+			}
 			if ( url_cmp_str(&m->map_url,&bw->route[0]) )
 				continue;
 			if ( url_cmp_str(&m->src_url,&bw->route[1]) )
@@ -172,10 +174,12 @@ int g_len,i;
 		}
 		else {
 		int len;
-			len = l_strlen(m->src_url.resource);
-			if ( l_strcmp(&m->src_url.resource[len-4],
-					l_string(std_cm,".crd")) == 0 )
-				continue;
+			if ( m->src_url.resource ) {
+				len = l_strlen(m->src_url.resource);
+				if ( l_strcmp(&m->src_url.resource[len-4],
+						l_string(std_cm,".crd")) == 0 )
+					continue;
+			}
 		}
 		lock_task(res_lock);
 		if ( m->flags & MF_BROWSE ) {
@@ -322,30 +326,14 @@ cnt = 0;
 			bw->load_max > 0 ; 
 			m = R_NEXT(MAP*,&m->h) ) {
 cnt ++;
-		len = l_strlen(m->src_url.resource);
-		if ( l_strcmp(&m->src_url.resource[len-4],
-				l_string(std_cm,".crd")) == 0 )
-			continue;
+		if ( m->src_url.resource ) {
+			len = l_strlen(m->src_url.resource);
+			if ( l_strcmp(&m->src_url.resource[len-4],
+					l_string(std_cm,".crd")) == 0 )
+				continue;
+		}
 
 		g = get_surp_rect(&g_len,&bw->surp,r);
-/*
-if ( l_strcmp(r->h.entry.resource,l_string(std_cm,"05.crd")) == 0 )
-ss_printf("***************************\n");
-ss_printf("CC / %ls\n",get_url_str2(&r->h.entry));
-ss_printf("CC-- %i %i ++ %i\n",m->cindex_time,m->mod_time,
-m->cindex_time - m->mod_time);
-*/
-/*
-		if ( m->cindex_time >= m->mod_time &&
-				m->cindex_mr.tl.x <= m->cindex_mr.br.x ) {
-			if ( cross_rect_rect(&m->cindex_mr,&g) == 0 ) {
-				lock_task(res_lock);
-				m->flags &= ~MF_BROWSE;
-				unlock_task(res_lock,"write_lock");
-				continue;
-			}
-		}
-*/
 		if ( m->cindex_mr.tl.x <= m->cindex_mr.br.x ) {
 			for ( i = 0 ; i < g_len ; i ++ ){
 				if ( cross_rect_rect(&m->cindex_mr,&g[i]) )
@@ -402,10 +390,12 @@ LOAD_RESOURCE_WORK * wp;
 			m != (MAP*)&r->c.map_children;
 			m = R_NEXT(MAP*,&m->h) ) {
 
-		len = l_strlen(m->src_url.resource);
-		if ( l_strcmp(&m->src_url.resource[len-4],
-				l_string(std_cm,".crd")) == 0 )
-			continue;
+		if ( m->src_url.resource ) {
+			len = l_strlen(m->src_url.resource);
+			if ( l_strcmp(&m->src_url.resource[len-4],
+					l_string(std_cm,".crd")) == 0 )
+				continue;
+		}
 
 		lock_task(res_lock);
 		if ( m->flags & MF_BROWSE ) {
@@ -554,7 +544,7 @@ int i;
 	for ( m = R_NEXT(MAP*,&r->c.map_children);
 			m != (MAP*)&r->c.map_children;
 			m = R_NEXT(MAP*,&m->h) ) {
-		{
+		if ( m->src_url.resource ) {
 		int len;
 			len = l_strlen(m->src_url.resource);
 			if ( l_strcmp(&m->src_url.resource[len-4],
@@ -651,10 +641,12 @@ MAP_HISTORY * mh;
 			m = R_NEXT(MAP*,&m->h) ) {
 		if ( bw->type == BWT_ROUTE ) {
 		int len;
-			len = l_strlen(m->dest_url.resource);
-			if ( l_strcmp(&m->dest_url.resource[len-4],
-					l_string(std_cm,".crd")) )
-				continue;
+			if ( m->dest_url.resource ) {
+				len = l_strlen(m->dest_url.resource);
+				if ( l_strcmp(&m->dest_url.resource[len-4],
+						l_string(std_cm,".crd")) )
+					continue;
+			}
 			if ( url_cmp_str(&m->map_url,&bw->route[0]) )
 				continue;
 			if ( url_cmp_str(&m->dest_url,&bw->route[1]) )
@@ -663,10 +655,12 @@ MAP_HISTORY * mh;
 		}
 		else {
 		int len;
-			len = l_strlen(m->dest_url.resource);
-			if ( l_strcmp(&m->dest_url.resource[len-4],
-					l_string(std_cm,".crd")) == 0 )
-				continue;
+			if ( m->dest_url.resource ) {
+				len = l_strlen(m->dest_url.resource);
+				if ( l_strcmp(&m->dest_url.resource[len-4],
+						l_string(std_cm,".crd")) == 0 )
+					continue;
+			}
 		}
 		lock_task(res_lock);
 		if ( m->flags & MF_BROWSE ) {
@@ -1012,14 +1006,16 @@ int proc;
 			continue;
 		}
 		if ( m1->src == 0 ) {
-			if ( l_strcmp(&m1->src_url.resource
+			if ( m1->src_url.resource && 
+				l_strcmp(&m1->src_url.resource
 					[l_strlen(m1->src_url.resource)-4],
 					l_string(std_cm,".crd")) )
 				proc = 2;
 			continue;
 		}
 		if ( m1->map_file == 0 ) {
-			if ( l_strcmp(&m1->src_url.resource
+			if ( m1->src_url.resource &&
+				l_strcmp(&m1->src_url.resource
 					[l_strlen(m1->src_url.resource)-4],
 					l_string(std_cm,".crd")) )
 				proc = 3;
