@@ -304,7 +304,7 @@ int target;
 	target = dm_nos * 0.75;
 	if ( target <= 0 )
 		target = 1;
-printf("scan %i\n",target);
+	log_printf(LOG_MESSAGE,LOG_LAYER_GB,0,"scan %i\n",target);
 	for ( ; check_queue(&pipe1,0,0) ; )
 		sleep_sec(1);
 
@@ -321,8 +321,14 @@ printf("scan %i\n",target);
 		lock_task(dm_lock);
 		d_st = d = R_PREV(DRAW_MATRIX *,&dm_h);
 		for ( ; ; ) {
-			if ( d == (DRAW_MATRIX*)&dm_h )
+			if ( d == (DRAW_MATRIX*)&dm_h ) {
 				d = R_PREV(DRAW_MATRIX*,&dm_h);
+				if ( d == d_st ) {
+					log_printf(LOG_WARNING,LOG_LAYER_GB,0,
+							"UNRESOLVE SCAN DM 1\n");
+					goto end;
+				}
+			}
 			if ( d->up == 0 )
 				goto no;
 
@@ -345,10 +351,11 @@ printf("scan %i\n",target);
 			break;
 		no:
 			d = R_PREV(DRAW_MATRIX*,&d->h);
-			if ( d == d_st )
-{ss_printf("UNRESOLVE SCAN DM\n");
+			if ( d == d_st ) {
+				log_printf(LOG_WARNING,LOG_LAYER_GB,0,
+						"UNRESOLVE SCAN DM 2\n");
 				goto end;
-}
+			}
 		}
 	}
 
@@ -400,7 +407,6 @@ set_cpu_thr_type(6);
 		for ( ; get_type(ld) == XLT_PAIR ; ld = cdr(ld) ) {
 			ret = _dgb_p2d_trailer(r,car(ld));
 			if ( get_type(ret) == XLT_ERROR ) {
-printf("ERROR 1\n");
 				set_inc_status_error(r,ret);
 			}
 		}
@@ -578,7 +584,7 @@ printf("get_handle_time = (%x) %i %i\n",dm,dm->get_handle_time,
 max_size);
 */
 	set_inc_status(r,RS_LOAD);
-	gc_push(0,0,"loading_draw");
+
 	gc_push(0,0,"loading_draw");
 	gt = get_symbol(l_string(std_cm,"Get"));
 
@@ -712,6 +718,8 @@ set_cpu_msg(2);
 	n->h.key = get_server_key(&r->h.target);
 
 	insert_queue(&pipe0,n,1);
+	
+	gc_pop(0,0);
 
 	return 0;
 }
@@ -739,6 +747,8 @@ set_cpu_thr_type(6);
 
 	for ( ; ; ) {
 
+		gc_push(0,0,"dm_loading_task0");
+
 		n1 = delete_queue(&pipe0,0,0,1);
 
 		ret = n1->data;
@@ -748,12 +758,9 @@ set_cpu_thr_type(6);
 
 		switch ( get_type(ret) ) {
 		case XLT_ERROR:
-			fprintf(stderr,"pdbp get error ");
-			print_sexp(s_stderr,ret,0);
-
+			log_printf(LOG_ERROR,LOG_LAYER_GB,0,"pdbp get error ");
 			set_inc_status_error(r,ret);
 
-			fprintf(stderr,"\n");
 			er = -1;
 			goto end;
 		case XLT_PAIR:
@@ -771,14 +778,14 @@ set_cpu_thr_type(6);
 					dm->get_handle_time = 1;
 				dm->get_level = n1->lev_max;
 				er = -2;
-printf("ERROR 3\n");
+				log_printf(LOG_ERROR,LOG_LAYER_GB,0,"ERROR 3\n");
 				goto end;
 			}
 			ret_query = get_el(ret,1);
 			break;
 		case XLT_NULL:
 			set_inc_status(r,RS_IDLE);
-printf("ERROR 4\n");
+			log_printf(LOG_ERROR,LOG_LAYER_GB,0,"ERROR 4\n");
 			er = -1;
 			goto end;
 		default:
@@ -814,8 +821,6 @@ printf("ERROR 4\n");
 
 		insert_queue(&pipe1,n2,1);
 	end:
-		gc_pop(0,0);
-		gc_pop(0,0);
 
 		lock_task(dm_lock);
 		if ( er == -2 )
@@ -831,6 +836,8 @@ printf("ERROR 4\n");
 
 		d_f_ree(n1->h.key);
 		d_f_ree(n1);
+		
+		gc_pop(0,0);
 	}
 }
 
@@ -1099,6 +1106,7 @@ set_cpu_thr_type(4);
 			unlock_task(dm_lock,"scan_Required");
 			continue;
 		}
+		gc_check_empty("dm_scan_task",0);
 		sleep_task((int)&dm_h,dm_lock);
 
 		continue;
