@@ -17,6 +17,7 @@
 
 
 #include	"task.h"
+#include	"memory_routine.h"
 
 extern SEM style_lock,dm_lock,radar_lock;
 
@@ -59,15 +60,35 @@ gc_view_unlock()
 void
 gc_view()
 {
+	GC_POSITION;
+
 	gc_lib();
+
+	GC_POSITION;
+
 	gc_setup();
+
+	GC_POSITION;
+
 	gc_gv_resource();
+
+	GC_POSITION;
+
 	gc_indicate();
 /*
 	gc_htmldb();
 */
+
+	GC_POSITION;
+
 	gc_dm();
+
+	GC_POSITION;
+
 	gc_style();
+
+	GC_POSITION;
+
 	gc_radar();
 }
 
