@@ -957,7 +957,7 @@ load_peripheral(L_CHAR * peripheral) {
 char * file;
 STREAM * st;
 XLISP_ENV * env;
-XL_SEXP * lst, * ret;
+XL_SEXP * lst,* ret;
 DIR * dd;
 struct dirent * d;
 
@@ -1000,6 +1000,11 @@ struct dirent * d;
 			sprintf(file,"%s%s",get_peripheral_path(),d->d_name);
 			env = new_env(gblisp_top_env1);
 			lst = load_file(env,0,2,l_string(std_cm,file),0,0,0,0,0);
+			if ( get_type(lst) == XLT_ERROR ) {
+				print_sexp(s_stderr,lst,0);
+				s_printf(s_stderr,"\n");
+			}
+/*
 			for ( ; get_type(lst) == XLT_PAIR ; lst = cdr(lst) ) {
 				gc_push(0,0,"loop2");
 				ret = eval(env,car(lst));
@@ -1011,6 +1016,7 @@ struct dirent * d;
 				}
 				gc_pop(0,0);
 			}
+*/
 		}
 
 		closedir(dd);
