@@ -494,12 +494,11 @@ ss_printf("** ** * RESET STREAM %i - %i\n",nr_count,wir);
 
 		switch ( get_type(n1->data) ) {
 		case XLT_ERROR:
-			fprintf(stderr,"r64 get error ");
-			print_sexp(s_stderr,n1->data,0);
+			log_print_sexp(LOG_MESSAGE,LOG_LAYER_GB,0,
+				"r64 get error",n1->data,0);
 
 			set_inc_status_error(r,n1->data);
 
-			fprintf(stderr,"\n");
 			goto n1_free;
 		case XLT_PAIR:
 			set_inc_status(r,RS_IDLE);
