@@ -231,7 +231,8 @@ GBVIEW_FLAME * gf;
 			if ( lock_flag == 0 )
 				unlock_task(gbview_flame_lock,"wf_apply_all_gf");
 			if ( (*func)(gf,work) < 0 ) {
-				lock_task(gbview_flame_lock);
+				if ( lock_flag == 0 )
+					lock_task(gbview_flame_lock);
 				break;
 			}
 			if ( lock_flag == 0 )
@@ -1052,10 +1053,13 @@ WIN_FLAME * wf;
 }
 
 void
-_free_win_flame(GBVIEW_FLAME * gf,WIN_FLAME * wf)
+_xx_free_win_flame(GBVIEW_FLAME * gf,WIN_FLAME * wf,char * __file,int __line)
 {
 	wf->flags |= WFF_FREE;
-ss_printf("FREE %ls\n",get_url_str2(&wf->draw->h.entry));
+	log_printf(LOG_MESSAGE,LOG_LAYER_GB,0,
+		"FREE(%s:%i) %ls\n",
+		__file,__line,
+		get_url_str2(&wf->draw->h.entry));
 	_wakeup_at_exit_task_wp(gf);
 	_wakeup_at_redraw_wp(gf);
 }
@@ -1090,7 +1094,7 @@ set_t_msg(19);
 }
 
 void
-free_win_flame(GBVIEW_FLAME * gf,WF_ID wfid)
+xx_free_win_flame(GBVIEW_FLAME * gf,WF_ID wfid,char * __file,int __line)
 {
 WIN_FLAME * wf;
 
@@ -1102,7 +1106,7 @@ set_t_msg(20);
 		wf_unlock(gf);
 		return;
 	}
-	_free_win_flame(gf,wf);
+	_xx_free_win_flame(gf,wf,__file,__line);
 	wf_unlock(gf);
 }
 
@@ -2919,8 +2923,6 @@ int nos;
 RESOURCE * map;
 WFID_INCLUDE_RESOURCE * wir,* p;
 	wir = wf_search_wfid_include_resource(r);
-if ( r->h.target.port == 9000 )
-ss_printf("********DIRTY 1 %x\n",wir);
 	for ( p = wir ; p ; p = p->next ) {
 		if ( p->id == 0 )
 			continue;
@@ -2945,12 +2947,6 @@ ss_printf("********DIRTY 1 %x\n",wir);
 		rct.tl.y -= ofs_h;
 		rct.br.x += ofs_w;
 		rct.br.y += ofs_h;
-if ( r->h.target.port == 9000 )
-ss_printf("********DIRTY %f %f - %f %f\n",
-rct.tl.x,
-rct.tl.y,
-rct.br.x,
-rct.br.y);
 		win_flame_dirty(p->gf,p->id,&rct,flags,0);
 
 		free_mh(mh);
