@@ -172,7 +172,7 @@ int i,j,k,sub;
 	else	start = n->mp_tbl[pos].crd;
 /*
 if ( localhost_check_str(start,"routing_optimize_el") < 0 ) {
-	log_printf(LOG_WARNING,"REMOTE_CK_OPT pos=%i local_ptr=%i\n",
+	log_printf(LOG_WARNING,LOG_LAYER_GB,0,"REMOTE_CK_OPT pos=%i local_ptr=%i\n",
 		pos,*local_ptr);
 }
 */
@@ -306,7 +306,7 @@ set_cpu_msg(720007);
 set_cpu_msg(0);
 
 	if ( get_type(ret_sexp) == XLT_ERROR ) {
-		log_print_sexp(LOG_WARNING,"err",ret_sexp,0);
+		log_print_sexp(LOG_WARNING,LOG_LAYER_GB,0,"err",ret_sexp,0);
 	} 
 
 	set_d_sexp(n->d,ret_sexp);
@@ -1034,8 +1034,8 @@ set_cpu_msg(720008);
 set_cpu_msg(0);
 
 	if ( get_type(ret_sexp) == XLT_ERROR ) {
-		log_print_sexp(LOG_WARNING,"cmd",s,0);
-		log_print_sexp(LOG_WARNING,"err",ret_sexp,0);
+		log_print_sexp(LOG_WARNING,LOG_LAYER_GB,0,"cmd",s,0);
+		log_print_sexp(LOG_WARNING,LOG_LAYER_GB,0,"err",ret_sexp,0);
 	} 
 
 	return ret_sexp;
