@@ -502,7 +502,7 @@ int len;
 L_CHAR * _f;
 	_f = get_url_filepath(u);
 	if ( path_check(_f) == 0 ) {
-		log_printf(LOG_DEBUG,"save_lr CHECK ERROR %ls\n",_f);
+		log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"save_lr CHECK ERROR %ls\n",_f);
 		d_f_ree(_f);
 		return;
 	}
@@ -852,7 +852,7 @@ unsigned int now;
 			info.crd = ll_copy_str(get_url_str2(&u1));
 			opt.lump = get_lump_path(new_lump());
 
-log_printf(LOG_DEBUG,"new lump = %s",n_string(std_cm,opt.lump));
+log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"new lump = %s",n_string(std_cm,opt.lump));
 
 			save_lump_info(opt.lump,&info);
 /*
@@ -865,7 +865,7 @@ printf("opt.lum1 = %s\n",n_string(std_cm,opt.lump));
 			path = get_lump_info_path(opt.lump);
 			fd = open(path,O_RDONLY);
 			if ( fd < 0 ) {
-log_printf(LOG_DEBUG,"cannot open the lump path %s",path);
+log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"cannot open the lump path %s",path);
 
 				d_f_ree(opt.lump);
 				d_f_ree(path);
