@@ -24,8 +24,12 @@
 #include	"mp.h"
 #include	"gbmp.h"
 
+
 int idle_interval;
 
+L_CHAR * get_full_url(L_CHAR * current,L_CHAR * url);
+L_CHAR * get_full_my_url(L_CHAR * current);
+
 
 void
 set_idle_interval(float p)
@@ -158,6 +162,74 @@ XL_SEXP * r;
 	return ret;
 }
 
+XL_SEXP *
+get_mapping_with_alias(int ses,L_CHAR * filename,L_CHAR ** ret_map)
+{
+XL_SEXP * meta;
+XL_SEXP * a, * a_sym;
+L_CHAR * t, * target;
+URL u;
+L_CHAR * f;
+XL_SEXP * ret;
+XL_SEXP * get_sym;
+
+	*ret_map = 0;
+	meta = get_mapping(filename,".mta");
+	switch ( get_type(meta) ) {
+	case XLT_ERROR:
+	case XLT_NULL:
+		return meta;
+	case XLT_PAIR:
+		break;
+	default:
+		return 0;
+	}
+	a = get_el_by_symbol(get_el(meta,1),
+		l_string(std_cm,"alias"),0);
+	if ( a == 0 ) {
+		*ret_map = get_full_my_url(filename);
+		return meta;
+	}
+	a_sym = car(a);
+	t = get_sf_attribute(a_sym->symbol.field,
+			l_string(std_cm,"target"));
+
+	if ( t == 0 )
+		return meta;
+	target = get_full_url(filename,t);
+	*ret_map = target;
+	get_url2(&u,target);
+
+	f = get_url_filepath(&u);
+
+	get_sym = n_get_symbol("Get");
+	set_attribute(get_sym,
+		l_string(std_cm,"mode"),
+		l_string(std_cm,"meta"));
+/*
+ss_printf("REMOTE_SESSION = %ls %ls\n",get_url_str2(&u),f);
+*/
+	ret = remote_session(
+		gblisp_top_env0,
+		ses,
+		&u,
+		l_string(std_cm,"gbstd"),
+		l_string(std_cm,"user"),
+		l_string(std_cm,"Get"),
+		List(
+			List(get_sym,
+				get_string(f),
+				-1),
+			-1),
+		0,0,0);
+
+	free_url(&u);
+	d_f_ree(f);
+
+	return ret;
+
+}
+
 int
 check_mapping(
 	L_CHAR * filename,
@@ -421,6 +493,9 @@ int len;
 	_target = d_re_alloc(_target,(len+10)*sizeof(L_CHAR));
 	l_strcpy(&_target[len],l_string(std_cm,prefix));
 	get_url2(&u,_target);
+/*
+ss_printf("SET_TARGET %ls\n",_target);
+*/
 	d_f_ree(_target);
 	ret = remote_session(
 		gblisp_top_env0,
@@ -457,12 +532,22 @@ int len;
 		0,0,0);
 	d_f_ree(f);
 	free_url(&u);
+/*
+ss_printf("++>");
+print_sexp(s_stdout,ret,0);
+ss_printf("\n");
+*/
 	if ( get_type(ret) == XLT_PAIR ) {
 		ret = get_el(ret,1);
 		if ( get_type(ret) != XLT_INTEGER )
 			return 0;
 		return ret->integer.data;
 	}
+	if ( get_type(ret) == XLT_ERROR ) {
+		log_print_sexp(LOG_WARNING,LOG_LAYER_GB,0,
+			"MP set_target ",
+			ret,0);
+	}
 	return 0;
 	
 }
@@ -640,7 +725,7 @@ ping_mapping(int ses,L_CHAR * filename,int force)
 XL_SEXP * xl_GetElement();
 XL_SEXP * meta;
 XL_SEXP * mpt;
-L_CHAR * src, * dest, * map;
+L_CHAR * src, * dest, * map,* _map;
 XL_SEXP * dp;
 int _dp;
 L_CHAR * target;
@@ -678,41 +763,76 @@ unsigned int mod;
 
 next:
 	mod = get_file_modify(filename,".mta");
-	meta = get_mapping(filename,".mta");
-	if ( meta == 0 ) {
-		log_printf(LOG_WARNING,"MP: cannot access metadata of the %s",
+	meta = get_mapping_with_alias(ses,filename,&map);
+	switch ( get_type(meta) ) {
+	case XLT_NULL:
+		log_printf(LOG_WARNING,LOG_LAYER_GB,0,
+			"MP: cannot access metadata of the %s",
 			n_string(std_cm,filename));
+		if ( map )
+			d_f_ree(map);
+		return -1;
+	case XLT_ERROR:
+		log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+			"MP: cannot access the meta data",meta,0);
+		if ( map )
+			d_f_ree(map);
 		return -1;
 	}
+
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+"MP access the meta data",meta,0);
+
 	meta = get_el(meta,1);
-	if ( meta == 0 )
+	if ( meta == 0 ) {
+		if ( map )
+			d_f_ree(map);
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+"MP access err 1",meta,0);
 		return interval;
-	get_sd_url(&src,&dest,&map,meta);
-	if ( src == 0 || dest == 0 )
+	}
+	get_sd_url(&src,&dest,&_map,meta);
+	if ( src == 0 || dest == 0 ) {
+		if ( map )
+			d_f_ree(map);
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+"MP access err 2",meta,0);
 		return interval;
+	}
 	dp = xl_GetElement(gblisp_top_env0,
 		List(n_get_symbol("GetElement"),
 			meta,
 			n_get_symbol("dp"),
 			-1));
-	if ( dp == 0 )
+	if ( dp == 0 ) {
+		if ( map )
+			d_f_ree(map);
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+"MP access err 3",meta,0);
 		return interval;
+	}
 	dp = get_el(dp,1);
-	if ( get_type(dp) != XLT_INTEGER )
+	if ( get_type(dp) != XLT_INTEGER ) {
+		if ( map )
+			d_f_ree(map);
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+"MP access err 4",meta,0);
 		return interval;
+	}
 
 
 	_dp = dp->integer.data;
 	src = get_full_url(filename,src);
 	dest = get_full_url(filename,dest);
-	map = get_full_my_url(filename);
+//	map = get_full_my_url(filename);
+
 	if ( force == 3 ) {
 		delete_target(ses,src,src,dest,map,_dp,".par");
 		delete_target(ses,dest,src,dest,map,_dp,".chi");
 		d_f_ree(src);
 		d_f_ree(dest);
 		d_f_ree(map);
-		log_printf(LOG_DEBUG,"MP: delete %s",
+		log_printf(LOG_DEBUG,LOG_LAYER_GB,0,"MP: delete %s",
 			n_string(std_cm,filename));
 		return -1;
 	}
@@ -730,6 +850,7 @@ next:
 		pmd_time = get_xltime() + pmd_interval;
 
 	}
+
 	if ( force >= 4 ) {
 
 		set_mapping(filename,time,interval,pmd_time,pmd_interval);
@@ -744,6 +865,8 @@ next:
 				if ( pmd_time > get_xltime() )
 					return pmd_time - get_xltime();
 			}
+log_print_sexp(LOG_DEBUG,LOG_LAYER_GB,0,
+"MP access err 6",meta,0);
 			if ( interval > pmd_interval )
 				return pmd_interval;
 			else	return interval;
