@@ -43,8 +43,8 @@ L_CHAR * unit_path;
 int len1,len2;
 
 
-printf("************\n");
-printf("db = %s (%f %f)-(%f %f) %f %s %s\n",
+log_printf(LOG_MESSAGE,LOG_LAYER_GB,0,
+"db = %s (%f %f)-(%f %f) %f %s %s\n",
 n_string(std_cm,db),
 r->h.minrect.tl.x,
 r->h.minrect.tl.y,
@@ -53,7 +53,6 @@ r->h.minrect.br.y,
 r->h.visible_resolution,
 n_string(std_cm,crd),
 n_string(std_cm,get_url_str2(&r->h.entry)));
-printf("+++++++++++++\n");
 
 
 	if ( r->h.minrect.br.x < r->h.minrect.tl.x )
