@@ -248,6 +248,7 @@ L_CHAR * _file;
 int p1;
 char buf[100];
 XL_INTERPRETER * xli;
+char * u_err;
 
 	_file = ll_copy_str(filename);
 	for ( p1 = l_strlen(_file)-1 ; p1 >= 0 ; p1 -- )
@@ -261,27 +262,46 @@ XL_INTERPRETER * xli;
 	get_url2(&u_src,src);
 	get_url2(&u_dest,dest);
 	get_url2(&u_map,map);
+
+	u_err = "src";
 	if ( u_src.proto == 0 || u_src.server == 0 ||
-			u_src.db == 0 || u_src.resource == 0 )
+			u_src.db == 0 )
 		goto url_error;
+
+	u_err = "src protocol";
 	if ( l_strcmp(u_src.proto,l_string(std_cm,"xlp")) )
 		goto url_error;
+
+	u_err = "src: not the direct path URL";
 	if ( u_src.db[0] != '/' )
 		goto url_error;
+
+	u_err = "dest";
 	if ( u_dest.proto == 0 || u_dest.server == 0 ||
 			u_dest.db == 0 || u_dest.resource == 0 )
 		goto url_error;
+
+	u_err = "dest protocol";
 	if ( l_strcmp(u_dest.proto,l_string(std_cm,"xlp")) )
 		goto url_error;
+
+	u_err = "dest: not the direct path URL";
 	if ( u_dest.db[0] != '/' )
 		goto url_error;
+
+	u_err = "map";
 	if ( u_map.proto == 0 || u_map.server == 0 ||
-			u_map.db == 0 || u_map.resource == 0 )
+			u_map.db == 0 )
 		goto url_error;
+
+	u_err = "map protocol";
 	if ( l_strcmp(u_map.proto,l_string(std_cm,"xlp")) )
 		goto url_error;
+
+	u_err = "map: not the direct path URL";
 	if ( u_map.db[0] != '/' )
 		goto url_error;
+
 	if ( l_strcmp(gf->mode,l_string(std_cm,"src")) == 0 )
 		u_me = &u_src;
 	else 	u_me = &u_dest;
@@ -310,6 +330,7 @@ XL_INTERPRETER * xli;
 	l_strcpy(&db_res[len],u_me->resource);
 	if ( l_strcmp(_file,db_res) ) {
 		d_f_ree(db_res);
+		u_err = "file != db_res";
 		goto url_error;
 	}
 	d_f_ree(db_res);
@@ -336,16 +357,24 @@ XL_INTERPRETER * xli;
 		goto url_permission;
 	}
 */
+
+
 	return 0;
 url_error:
+
+
 	ret = get_error(
 		s->h.file,
 		s->h.line,
 		XLE_PROTO_INV_PARAM,
 		l_string(std_cm,"Set"),
-		n_get_string("invalid parameter in URL"));
+		List(n_get_string("invalid parameter in URL"),
+			n_get_string(u_err),
+			-1));
 	goto err;
 url_permission:
+
+
 	ret = get_error(
 		s->h.file,
 		s->h.line,
@@ -356,6 +385,8 @@ url_permission:
 			-1));
 	goto err;
 get_user_type_missmatch:
+
+
 	ret = get_error(
 		s->h.file,
 		s->h.line,
@@ -365,6 +396,8 @@ get_user_type_missmatch:
 		"type missmatch of the return value of GetUserInfo"));
 	goto err;
 type_missmatch:
+
+
 	ret = get_error(
 		s->h.file,
 		s->h.line,
@@ -453,6 +486,7 @@ XL_SEXP * _interval;
 int trigger;
 int mf,mf2,mf3;
 int ll;
+URL uu;
 
 int db_f;
 
@@ -513,6 +547,12 @@ int db_f;
 				option = OP_DELETE;
 		}
 	}
+	get_url2(&uu,_filename);
+	if ( uu.agent || uu.name || uu.query ) {
+		free_url(&uu);
+		goto end;
+	}
+	free_url(&uu);
 	target[0] = target[1] = 0;
 	fn = nl_copy_str(std_cm,"Set");
 	ret = get_path(target,&gf,_filename,s,fn);
@@ -529,6 +569,7 @@ int db_f;
 		if ( get_type(ret) == XLT_ERROR )
 			goto end;
 	}
+
 	rec = normalize_rec(gf->mode,rec);
 	if ( env )
 		rec = cons(car(rec),
@@ -548,21 +589,27 @@ int db_f;
 	else	mf2 = 0;
 
 	for ( i = 0 ; i < 2 ; i ++ ) {
+
 		if ( target[i] == 0 )
 			break;
 		lock_path = 0;
+
 		switch ( gf->flags & XLGFM_LOCK ) {
 		case XLGF_FULL_PATH:
+
 			lr = call_lock(lock_path = target[i],
 				       CLT_WRITE_LOCK);
+
 			if ( cl_error_check(lr) ) {
 				ret = get_cl_error(s,"Set");
 				goto end;
 			}
 			break;
 		case XLGF_URL_PATH:
+
 			lr = call_lock(lock_path=_filename,
 					CLT_WRITE_LOCK);
+
 			if ( cl_error_check(lr) ) {
 				ret = get_cl_error(s,"Set");
 				goto end;
@@ -571,9 +618,11 @@ int db_f;
 		}
 		access_path = target[i];
 		ret = load_file(env,s,1,target[i],0,0,0,0,0);
+
 		if ( get_type(ret) != XLT_ERROR )
 			break;
 		call_unlock(lr);
+
 		if ( ret->err.code != XLE_PROTO_OPEN_FILE )
 			goto end;
 	}
@@ -583,8 +632,8 @@ int db_f;
 	ret = 0;
 	replace_flag = 0;
 
-
 	for ( a = load_data ; get_type(a) ; a = cdr(a) ) {
+
 		b = car(a);
 		if ( replace_flag ) {
 			ret = cons(b,ret);
@@ -660,6 +709,7 @@ int db_f;
 		replace_flag = 1;
 		trigger = 1;
 	}
+
 	if ( (option == OP_INSERT ||
 			option == OP_PART) &&
 			replace_flag == 0 ) {
@@ -696,6 +746,7 @@ int db_f;
 close_end:
 	call_unlock(lr);
 end:
+
 	if ( target[0] )
 		d_f_ree(target[0]);
 	if ( target[1] )
@@ -712,11 +763,12 @@ ss_printf("MF %i %i %ls %ls %ls\n",mf,mf2,_filename,src->string.data,dest->strin
 			send_cindex(_filename);
 	}
 	if ( get_type(ret) == XLT_ERROR ) {
-		log_print_sexp(LOG_ERROR,"Set operation error",
+		log_print_sexp(LOG_ERROR,LOG_LAYER_GB,0,"Set operation error",
 			ret,0);
 	}
 	return ret;
 type_missmatch:
+
 	return get_error(
 		s->h.file,
 		s->h.line,
@@ -724,6 +776,7 @@ type_missmatch:
 		l_string(std_cm,"Set"),
 		0);
 param_error:
+
 	return get_error(
 		s->h.file,
 		s->h.line,
@@ -872,7 +925,9 @@ XL_SEXP * ret;
 			s->h.line,
 			XLE_PROTO_UNDEF_RESOURCE,
 			l_string(std_cm,"Set"),
-			n_get_string("undefined prefix"));
+			List(n_get_string("undefined prefix"),
+				filename,
+				-1));
 	if ( l_strcmp(gf->mode,l_string(std_cm,"file")) == 0 )
 		ret = xl_Set_full(env,s);
 	else	ret = xl_Set_part(env,s,a_env,sf,0,0);
