@@ -29,7 +29,8 @@ typedef struct freelist {
 unsigned int alloc_filespace(PDB *,PN_HEADER * h);
 void free_filespace(PDB *,unsigned int);
 void * get_file_record(unsigned int *,PDB *,short);
-void * read_filespace(PDB *,unsigned int);
+void * xx_read_filespace(PDB *,unsigned int,char * __file,int __line);
+#define read_filespace(p,ofs)	xx_read_filespace((p),(ofs),__FILE__,__LINE__)
 int write_filespace(PDB *,void *);
 void flush_filespace(PDB * );
 
