@@ -77,9 +77,11 @@ delete_queue(SYS_QUEUE * q,int (*cond)(),void * work,int wait_flag);
 L_CHAR * touch_qkey(SYS_QUEUE * q);
 void release_qkey(SYS_QUEUE * q,L_CHAR * key);
 int sq_key_cond(SYS_QUEUE *,void *,void *);
-L_CHAR *  get_server_key(URL * u);
+L_CHAR *  xx_get_server_key(URL * u,char *,int);
+#define get_server_key(u)	xx_get_server_key((u),__FILE__,__LINE__)
 int check_queue(SYS_QUEUE * q,int (*cond)(),void * work);
-void * new_queue_node(int);
+void * xx_new_queue_node(int,char*,int);
+#define new_queue_node(s)	xx_new_queue_node((s),__FILE__,__LINE__)
 int get_active_que_thread(SYS_QUEUE*);
 
 #endif
