@@ -24,14 +24,18 @@
 #include	"long_char.h"
 #include	"xllisp.h"
 #include	"u_file.h"
+#include	"log.h"
 
 
 typedef struct url {
 	L_CHAR *		proto;
 	L_CHAR *		server;
 	int			port;
+	L_CHAR *		agent;
 	L_CHAR *		db;
 	L_CHAR *		resource;
+	L_CHAR *		query;
+	L_CHAR *		name;
 
 	L_CHAR *		user;
 	L_CHAR *		passwd;
@@ -72,6 +76,11 @@ typedef struct nested_lock {
 	int		cnt;
 } NESTED_LOCK;
 
+typedef struct attr_table {
+	char *		flag_name;
+	int		flag;
+} ATTR_TABLE;
+
 void init_thread_area(THREAD_AREA * ta,THREAD_AREA * ta_param);
 void * get_my_area(THREAD_AREA *);
 void close_thread_area();
@@ -83,15 +92,16 @@ char * xx_copy_str(char *,char *,int);
 
 L_CHAR * xx_ll_copy_str(L_CHAR *,char *,int);
 #define ll_copy_str(p)	xx_ll_copy_str((p),__FILE__,__LINE__)
-L_CHAR * nl_copy_str(CODE_METHOD *,char *);
+L_CHAR * xx_nl_copy_str(CODE_METHOD *,char *,char*,int);
+#define nl_copy_str(cm,p)	xx_nl_copy_str((cm),(p),__FILE__,__LINE__)
 char * ln_copy_str(CODE_METHOD *,L_CHAR*);
 int get_url(URL *,L_CHAR *);
-int get_url2(URL *,L_CHAR *);
+int xx_get_url2(URL *,L_CHAR *,char*,int);
+#define get_url2(u,str)		xx_get_url2(u,str,__FILE__,__LINE__)
 int url_cmp(URL *,URL *);
 int url_cmp_str(URL*,URL*);
 void zero_url(URL *);
 void free_url(URL *);
-L_CHAR * get_url_str(URL *);
 L_CHAR * get_url_str2(URL *);
 L_CHAR * get_url_filepath(URL *);
 int get_wkp(L_CHAR *);
@@ -172,4 +182,9 @@ char * normalize_delim_str(char * buf);
 
 void memory_watchdog(void(*)());
 
+int get_attr_flags(int *,L_CHAR *,ATTR_TABLE *);
+
+L_CHAR *xx_safety_string(L_CHAR * str,char * __file,int __line);
+#define safety_string(str) xx_safety_string(str,__FILE__,__LINE__);
+
 #endif
