@@ -369,5 +369,8 @@ int globe_move_plot(GBVIEW_FLAME * gf,
 
 int _e2d_p_redraw(GBVIEW_FLAME * gf,WIN_FLAME * wf);
 
+void _xx_free_win_flame(GBVIEW_FLAME * gf,WIN_FLAME*wf,char * __file,int __line);
+#define _free_win_flame(gf,wf)	_xx_free_win_flame((gf),(wf),__FILE__,__LINE__)
+
 
 #endif
