@@ -16,22 +16,35 @@
 **********************************************************************/
 
 
-
+#include	"utils.h"
 #include	"long_char.h"
 
 extern CODE_METHOD euc_cm, jis_cm,sjis_cm;
 
 
 CODE_METHOD *
-get_encoding(L_CHAR * code)
+get_encoding(L_CHAR * _code)
 {
-	if ( l_strcmp(code,l_string(std_cm,"Shift_JIS")) == 0 )
-		return &sjis_cm;
+L_CHAR * code;
+CODE_METHOD * ret;
+L_CHAR * p;
+	if ( _code == 0 )
+		return 0; 
+	code = ll_copy_str(_code);
+	for ( p = code ; *p ; p ++ ) {
+		if ( 'a' <= *p && *p <= 'z' )
+			*p += 'A' - 'a';
+	}
+	if ( l_strcmp(code,l_string(std_cm,"SHIFT_JIS")) == 0 )
+		ret = &sjis_cm;
+	else if ( l_strcmp(code,l_string(std_cm,"SHIFT-JIS")) == 0 )
+		ret = &sjis_cm;
 	else if ( l_strcmp(code,l_string(std_cm,"EUC")) == 0 )
-		return &euc_cm;
+		ret = &euc_cm;
 	else if ( l_strcmp(code,l_string(std_cm,"EUC-JP")) == 0 )
-		return &euc_cm;
+		ret = &euc_cm;
 	else if ( l_strcmp(code,l_string(std_cm,"JIS")) == 0 )
-		return &jis_cm;
-	return 0;
+		ret = &jis_cm;
+	else	ret = 0;
+	return ret;
 }
\ No newline at end of file
