@@ -20,14 +20,18 @@
 #define DONT_USE_LOCK
 */
 #include	<stdlib.h>
+#include	"log.h"
 #include	"memory_debug.h"
 #include	"lock_level.h"
 #include	"save_global.h"
+#include	"avt.h"
 
 #ifndef DONT_USE_LOCK
 #include	"task.h"
 #endif
 
+#define LOG_MEM_LIST_LEN	20
+
 
 
 #ifdef MEMORY_DEBUG
@@ -275,7 +279,6 @@ int large;
 int i;
 	LOCK;
 	large = 0;
-	printf("IND_MEM\n");
 	for ( i = 0; i < MEM_LIST_HASH_SIZE ; i ++ ) {
 		ml = mem_list_hash[i];
 		for ( ; ml ; ml = ml->next ) {
@@ -292,20 +295,90 @@ int i;
 	UNLOCK;
 }
 
+
+typedef struct log_mem {
+	int		size;
+	AVT_NODE *	root;
+	AVT_NODE	n[LOG_MEM_LIST_LEN];
+} LOG_MEM;
+
+
+int
+lm_cmp(MEM_LIST * m1,MEM_LIST * m2)
+{
+	if ( m1->size < m2->size )
+		return -1;
+	if ( m1->size > m2->size )
+		return 1;
+	if ( ((long)m1) < ((long)m2) )
+		return -1;
+	if ( ((long)m1) > ((long)m2) )
+		return 1;
+	return 0;
+}
+
+int
+lm_trace(AVT_NODE * a,AVT_NODE ** ret)
+{
+	*ret = a;
+	return 1;
+}
+
+void
+insert_lm(LOG_MEM * lm,MEM_LIST * memlist)
+{
+AVT_NODE * n;
+	if ( lm->size >= LOG_MEM_LIST_LEN ) {
+		avt_trace_from_small(lm->root,lm_trace,&n);
+		n = avt_delete(&lm->root,n->data,lm_cmp);
+	}
+	else {
+		n = &lm->n[lm->size++];
+	}
+	n->data = memlist;
+	avt_insert(&lm->root,n,lm_cmp);
+}
+
+int
+lm_log_output(AVT_NODE * a,int * cnt)
+{
+MEM_LIST * ml;
+	ml = a->data;
+	log_printf(LOG_DEBUG,LOG_LAYER_LIB,0,"::MEM_%i %s:%i / size=%i\n",
+		(*cnt)++,
+		ml->file,
+		ml->line,
+		ml->size);
+	return 0;
+}
+
+
 void
 list_tick()
 {
 MEM_LIST * ml;
 int large;
 int i;
+char * buf;
+int all_size;
+
+LOG_MEM * lm;
+//extern int obj_mem_size;
+
+	lm = d_alloc(sizeof(*lm));
+	lm->size = 0;
+	lm->root = 0;
 	LOCK;
 	large = 0;
 	mem_max_list = 0;
+	all_size = 0;
 	for ( i = 0; i < MEM_LIST_HASH_SIZE ; i ++ ) {
 		ml = mem_list_hash[i];
 		for ( ; ml ; ml = ml->next ) {
 			if ( ml->size == 0 )
 				continue;
+			insert_lm(lm,ml);
+			all_size += ml->size;
 			if ( large < ml->size ) {
 				large = ml->size;
 				ml->max_next = mem_max_list;
@@ -313,7 +386,33 @@ int i;
 			}
 		}
 	}
+	if ( mem_max_list ) {
+		buf = malloc(1000);
+		sprintf(buf,"MAX MEMORY = %s:%i / size= %i [TOTAL %i]\n",
+			mem_max_list->file,
+			mem_max_list->line,
+			mem_max_list->size,
+			all_size);
+	}
+	else buf = 0;
 	UNLOCK;
+
+	if ( buf ) {
+		i = 0;
+		log_printf(LOG_DEBUG,LOG_LAYER_LIB,0,"%s",buf);
+		free(buf);
+		avt_trace_from_large(lm->root,lm_log_output,&i);
+
+		log_printf(LOG_DEBUG,LOG_LAYER_LIB,0,
+				"IMPREMENTED MEMORY SIZE = %i\n",
+					get_mem_size());
+//		log_printf(LOG_DEBUG,LOG_LAYER_LIB,0,
+//				"OBJECT MEMEORY SIZE = %i\n",obj_mem_size);
+
+		log_printf(LOG_DEBUG,LOG_LAYER_LIB,0,"MEM_END\n");
+	}
+	
+	d_f_ree(lm);
 }
 
 MEM_LIST *
