@@ -270,6 +270,7 @@ int er;
 	if ( ret->f.h.type != PNT_FREELIST_HEADER &&
 			ret->f.h.type != PNT_FREELIST_NODE )
 		er_panic("next_freelist(2)");
+	f->next = ret;
 	ret->next = 0;
 	ret->fofs = f->f.next;
 	return ret;
@@ -586,13 +587,13 @@ err:
 
 
 void *
-read_filespace(PDB * p,unsigned int fofs)
+xx_read_filespace(PDB * p,unsigned int fofs,char * __file,int __line)
 {
 ACC_PN_HEADER * h;
 int er;
 unsigned short size;
 
-	h = d_alloc(sizeof(*h));
+	h = xx_d_alloc(sizeof(*h),__file,__line);
 	h->a.fofs = fofs;
 	u_lseek(p->fid,fofs,SEEK_SET);
 	er = u_read(p->fid,&h->h,sizeof(h->h));
