@@ -374,7 +374,8 @@ char * v, * r;
 	info->content_length = -1;
 	info->err = 200;
 	info->flags = 0;
-
+	info->charset = 0;
+	info->content_type = 0;
 
 	if ( flag == 0 )
 		goto next;
@@ -449,6 +450,26 @@ next:
 			sscanf(ret->str,"%i",&info->err);
 			free_GET_BUF(ret);
 		}
+		else if ( strcasecmp(ret->str,"Content-Type:") == 0 ) {
+		char * p;
+			ret = step_GET_BUF(ret);
+			info->content_type = ret->str;
+			ret->str = 0;
+			for ( p = info->content_type ; *p && *p != ';' ; p ++ );
+			*p = 0;
+			ret = step_GET_BUF(ret);
+			if ( ret ) {
+				for ( p = ret->str ; *p && *p != '=' ; p ++ );
+				if ( *p == 0 ) {
+					info->charset = ret->str;
+					ret->str = 0;
+				}
+				else {
+					info->charset = copy_str(p+1);
+				}
+				free_GET_BUF(ret);
+			}
+		}
 		else {
 			free_GET_BUF(ret);
 		}
