@@ -77,7 +77,7 @@ put_lstr(S_PRINTF_STREAM *s, L_CHAR *lstr){
 
 
 int 
-_s_printf(STREAM *s, char * fmt, va_list p){
+_s_printf(STREAM *s,int * erp, char * fmt, va_list p){
 	S_PRINTF_STREAM out;
 	char *in_begin;
 	char *in_end;
@@ -216,10 +216,14 @@ _s_printf(STREAM *s, char * fmt, va_list p){
 
 		lock_task(stream_lock);
 		if ( s == 0 ) {
+			if ( erp )
+				*erp = ESYS_INVAL;
 			unlock_task(stream_lock,"s_write");
 			return -1;
 		}
 		if ( s->h.tbl == 0 ) {
+			if ( erp )
+				*erp = ESYS_INVAL;
 			unlock_task(stream_lock,"s_write");
 			return -1;
 		}
@@ -281,13 +285,15 @@ _s_printf(STREAM *s, char * fmt, va_list p){
 				ss = ptr;
 			retry:
 				er = (*tbl->write)(s,pp,ss);
-				if ( er <= 0 ) {
+				if ( er < 0 ) {
 					if ( errno == ESYS_AGAIN ) {
 						ss = ss/2;
 						if ( ss <= 0 )
 							ss = 1;
 						goto retry;
 					}
+					if ( erp )
+						*erp = errno;
 					break;
 				}
 				if ( er == 0 )
@@ -327,7 +333,7 @@ int ret;
 #else
 	va_start(p,fmt);
 #endif
-	ret = _s_printf(s, fmt, p);
+	ret = _s_printf(s,0, fmt, p);
 	
 	va_end(p);
 	return ret;
@@ -351,7 +357,34 @@ STREAM * s;
 	va_start(p,fmt);
 #endif
 
-	ret = _s_printf(s, fmt, p);
+	ret = _s_printf(s,0, fmt, p);
+	
+	va_end(p);
+	return ret;
+}
+
+int
+e_printf(STREAM * s,int * erp,char * fmt,...)
+{
+va_list	p;
+int ret;
+
+	if ( s == 0 ) {
+		if ( erp )
+			*erp = ESYS_INVAL;
+		return -1; 
+	}
+	if ( s->h.tbl == 0 ) {
+		if ( erp )
+			*erp = ESYS_INVAL;
+		return -1;
+	}
+#ifdef VA2
+	va_start(p);
+#else
+	va_start(p,fmt);
+#endif
+	ret = _s_printf(s,erp, fmt, p);
 	
 	va_end(p);
 	return ret;
