@@ -31,6 +31,8 @@ L_CHAR * ptr;
 char port_buf[20];
 	buf = d_alloc(
 		((u->db ? l_strlen(u->db) : 0) +
+		 (u->name ? l_strlen(u->db) : 0) +
+		 (u->query ? l_strlen(u->db) : 0) +
 		 (u->resource ? l_strlen(u->resource) : 0) + 100) *
 			sizeof(L_CHAR));
 	buf[0] = 0;
@@ -48,8 +50,23 @@ db:
 	if ( *(ptr-1) != '/' )
 		*ptr ++ = '/';
 resource:
-	if ( u->resource )
+	*ptr = 0;
+	if ( u->resource ) {
 		l_strcpy(ptr,u->resource);
+		ptr = seek_ptr(ptr);
+	}
+query:
+	if ( u->query ) {
+		*ptr ++ = '?';
+		l_strcpy(ptr,u->query);
+		ptr = seek_ptr(ptr);
+	}
+	if ( u->name ) {
+		*ptr ++ = '#';
+		l_strcpy(ptr,u->name);
+		ptr = seek_ptr(ptr);
+	}
+
 	buf = d_re_alloc(buf,(l_strlen(buf)+1)*sizeof(L_CHAR));
 	return buf;
 err:
