@@ -39,6 +39,9 @@ char port_buf[20];
 		((u->proto ? l_strlen(u->proto) : 0) +
 		 (u->server ? l_strlen(u->server) : 0) +
 		 (u->db ? l_strlen(u->db) : 0) +
+		 (u->agent ? l_strlen(u->agent) : 0) +
+		 (u->query ? l_strlen(u->query) : 0) +
+		 (u->name ? l_strlen(u->name) : 0) +
 		 (u->resource ? l_strlen(u->resource) : 0) + 100) *
 			sizeof(L_CHAR));
 	buf[0] = 0;
@@ -51,15 +54,24 @@ char port_buf[20];
 	*ptr ++ = '/';
 	*ptr ++ = '/';
 	if ( u->server == 0 )
-		goto db;
+		goto agent;
 	l_strcpy(ptr,u->server);
 	ptr = seek_ptr(ptr);
 port:
 	if ( u->port == get_wkp(u->proto) )
-		goto db;
+		goto agent;
 	sprintf(port_buf,":%i",u->port);
 	l_strcpy(ptr,l_string(std_cm,port_buf));
 	ptr = seek_ptr(ptr);
+agent:
+	if ( u->agent == 0 )
+		goto db;
+	*ptr ++ = '/';
+	*ptr ++ = '@';
+	l_strcpy(ptr,u->agent);
+	ptr = seek_ptr(ptr);
+	if ( u->db && u->db[0] != '/' )
+		*ptr ++ = '/';
 db:
 	if ( u->db == 0 ) {
 		*ptr ++ = '/';
@@ -73,8 +85,21 @@ db:
 	if ( *(ptr-1) != '/' )
 		*ptr ++ = '/';
 resource:
-	if ( u->resource )
+	if ( u->resource ) {
 		l_strcpy(ptr,u->resource);
+		ptr = seek_ptr(ptr);
+	}
+	if ( u->query ) {
+		*ptr ++ = '?';
+		l_strcpy(ptr,u->query);
+		ptr = seek_ptr(ptr);
+	}
+	if ( u->name ) {
+		*ptr ++ = '#';
+		l_strcpy(ptr,u->name);
+		ptr = seek_ptr(ptr);
+	}
+
 	buf = d_re_alloc(buf,(l_strlen(buf)+1)*sizeof(L_CHAR));
 	set_buffer(buf);
 	return buf;
