@@ -21,9 +21,10 @@
 #include	"memory_debug.h"
 
 L_CHAR *
-nl_copy_str(CODE_METHOD * cm,char * str)
+xx_nl_copy_str(CODE_METHOD * cm,char * str,
+	char * __file,int __line)
 {
 L_CHAR * ret;
-	ret = ll_copy_str(l_string(cm,str));
+	ret = xx_ll_copy_str(l_string(cm,str),__file,__line);
 	return ret;
 }
