@@ -45,9 +45,13 @@ unsigned char emp_tag_table[][10] = {
 	"META",
 	"br",
 	"BR",
+	"hr",
+	"HR",
 	"!DOCTYPE",
 	"IMG",
 	"img",
+	"input",
+	"INPUT",
 	""
 };
 
@@ -811,10 +815,6 @@ XL_SYMBOL * s;
 int ret;
 int len;
 
-if ( memcmp(f->yytext,l_string(std_cm,"</Res"),5*4) == 0 &&
-l_strcmp(f->yytext,l_string(std_cm,"</Result>")) )
-printf("****************************** %s\n",n_string(std_cm,f->yytext));
-
 	if ( f->yytextlen <= f->yylen ) {
 		f->yytext = mrealloc(f->yytext,
 			(f->yylen+1)*sizeof(L_CHAR),
@@ -884,6 +884,12 @@ printf("****************************** %s\n",n_string(std_cm,f->yytext));
 		gc_text);
 	s = &get_sexp(f,XLT_SYMBOL)->symbol;
 	s->data = f->yytext;
+	if ( f->flags & XLF_HTML ) {
+	L_CHAR * p;
+		for ( p = s->data ; *p ; p ++ )
+			if ( 'a' <= *p && *p <= 'z' )
+				*p += 'A' - 'a';
+	}
 	s->h.flags |= GBF_XML;
 	f->yytext = 0;
 	f->yylen = 0; // *
@@ -932,6 +938,12 @@ int ret;
 	}
 	s = &get_sexp(f,XLT_SYMBOL)->symbol;
 	s->data = f->yytext;
+	if ( f->flags & XLF_HTML ) {
+	L_CHAR * p;
+		for ( p = s->data ; *p ; p ++ )
+			if ( 'a' <= *p && *p <= 'z' )
+				*p += 'A' - 'a';
+	}
 	s->h.flags |= GBF_XML;
 	f->yytext = 0;
 	f->yylen = 0; // *
@@ -947,6 +959,12 @@ XL_STRING * s;
 		gc_text);
 	f->yytext[f->yylen] = 0;
 	s->data = f->yytext;
+	if ( f->flags & XLF_HTML ) {
+	L_CHAR * p;
+		for ( p = s->data ; *p ; p ++ )
+			if ( 'a' <= *p && *p <= 'z' )
+				*p += 'A' - 'a';
+	}
 	s->h.flags |= GBF_XML;
 	f->yytext = 0;
 	f->yylen = 0; // *
@@ -1568,6 +1586,13 @@ p_x_tag:
 			f->line ++;
 		f->mode = XLM_X_TWAIT;
 		return get_x_tag(f);
+	case '-':
+		put_data(f,ch);
+		if ( f->yylen == 4 &&
+			memcmp(f->yytext,l_string(std_cm,"<!--"),
+				4*sizeof(L_CHAR)) == 0 )
+			return get_x_tag(f);
+		goto gb_read;
 	default:
 		put_data(f,ch);
 		goto gb_read;
@@ -2273,7 +2298,7 @@ p_l_field_name:
 		goto err;
 	}
 p_l_field_data:
-	t = _gb_lex(f,EST_ENTITY);
+	t = _gb_lex(f,EST_REPLACE);
 	if ( t != T_SEXP ) {
 		msg = l_string(std_cm,
 			"text is required 2");
