@@ -1095,7 +1095,7 @@ int ret;
 
 
 int
-get_delay_info(STREAM ** st,XL_DELAY ** buf,XL_SEXP * s)
+get_delay_info(STREAM ** st,DELAY_FUNC ** buf,XL_SEXP * s)
 {
 int t;
 XL_FILE * f,* f2;
