@@ -18,8 +18,16 @@
 
 #define STREAM_LIB
 
+#ifdef VA2
+#include	<varargs.h>
+#else
+#include	<stdarg.h>
+#endif
+#include	"xl.h"
+
 #include	<stdlib.h>
 #include	<errno.h>
+#include	<setjmp.h>
 #include	"machine/err.h"
 #include	"xl.h"
 #include	"memory_debug.h"
@@ -33,15 +41,40 @@ typedef struct indent {
 } INDENT;
 typedef void (*pt_t[2])();
 
-void
-_print_sexp(STREAM * fd,XL_SEXP * s,int flags,INDENT ind);
+typedef struct err_env {
+	jmp_buf		env;
+	int		er;
+} ERR_ENV;
 
 void
-print_ind(STREAM * fd,INDENT ind)
+_print_sexp(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags,INDENT ind);
+
+
+void
+_e_printf(STREAM * fd,ERR_ENV * ee,char * fmt,...)
+{
+va_list p;
+int ret;
+
+#ifdef VA2
+	va_start(p);
+#else
+	va_start(p,fmt);
+#endif
+	ret = _s_printf(fd,&ee->er,fmt,p);
+
+	va_end(p);
+
+	if ( ret < 0 )
+		longjmp(ee->env,-1);
+}
+
+void
+print_ind(STREAM * fd,ERR_ENV * ee,INDENT ind)
 {
 int i;
 	for ( i = 0 ; i < ind.ind ; i ++ )
-		s_printf(fd," ");
+		_e_printf(fd,ee," ");
 }
 
 int
@@ -103,11 +136,11 @@ check_valid_string(L_CHAR * str)
 }
 
 void
-_print_null_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_null_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 	if ( flags & PF_TEXT )
 		return;
-	s_printf(fd,"()");
+	_e_printf(fd,ee,"()");
 }
 
 pt_t pt_null = {
@@ -116,7 +149,7 @@ pt_t pt_null = {
 };
 
 void
-_print_error_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_error_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 INDENT ind;
 L_CHAR nl[1];
@@ -127,7 +160,7 @@ L_CHAR * site, * filename,* func;
 	ind.flags = 0;
 	ind.ind = 0;
 	if ( !(flags&PF_LISP_MODE) )
-		s_printf(fd,"^");
+		_e_printf(fd,ee,"^");
 
 	if ( s->err.site == 0 )
 		site = nl;
@@ -138,15 +171,15 @@ L_CHAR * site, * filename,* func;
 	if ( s->err.func == 0 )
 		func = nl;
 	else	func = s->err.func;
-	s_printf(fd,"\n %%E\n (\"%ls\" \"%ls\" %i \"%ls\" 0x%x\n ",
+	_e_printf(fd,ee,"\n %%E\n (\"%ls\" \"%ls\" %i \"%ls\" 0x%x\n ",
 		site,
 		filename,
 		s->err.line,
 		func,
 		s->err.code);
-	_print_sexp(fd,s->err.data,
+	_print_sexp(fd,ee,s->err.data,
 		(flags&(~(PFM_FORMAT|PF_TEXT)))|PF_LISP|PF_LISP_MODE,ind);
-	s_printf(fd,")");
+	_e_printf(fd,ee,")");
 }
 
 pt_t pt_error = {
@@ -155,7 +188,7 @@ pt_t pt_error = {
 };
 
 void
-_print_pair_l(STREAM * fd,XL_SEXP * s,int flags,INDENT ind)
+_print_pair_l(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags,INDENT ind)
 {
 int t;
 int f;
@@ -163,14 +196,14 @@ INDENT ind2;
 XL_SEXP * r;
 	if ( flags&PF_INDENT ) {
 		if ( ind.flags & IF_FORCE ) {
-			s_printf(fd,"\n");
-			print_ind(fd,ind);
+			_e_printf(fd,ee,"\n");
+			print_ind(fd,ee,ind);
 		}
 	}
 	ind2.flags = 0;
 	ind2.ind = ind.ind+1;
 	if ( !(flags & PF_TEXT) )
-		s_printf(fd,"(");
+		_e_printf(fd,ee,"(");
 	f = 0;
 	for ( ; (t=get_type(s)) == XLT_PAIR ; ) {
 /*
@@ -181,26 +214,26 @@ XL_SEXP * r;
 		} 
 */
 		if ( f )
-			s_printf(fd," ");
+			_e_printf(fd,ee," ");
 		f = 1;
-		_print_sexp(fd,car(s),flags|PF_LISP_MODE,ind2);
+		_print_sexp(fd,ee,car(s),flags|PF_LISP_MODE,ind2);
 		ind2.flags |= IF_FORCE;
 		s = cdr(s);
 	}
 	if ( t == 0 ) {
 		if ( !(flags & PF_TEXT) )
-			s_printf(fd,")");
+			_e_printf(fd,ee,")");
 	}
 	else {
-		s_printf(fd,".");
+		_e_printf(fd,ee,".");
 		ind2.flags |= IF_FORCE;
-		_print_sexp(fd,s,flags|PF_LISP_MODE,ind2);
-		s_printf(fd,")");
+		_print_sexp(fd,ee,s,flags|PF_LISP_MODE,ind2);
+		_e_printf(fd,ee,")");
 	}
 }
 
 void
-_print_field_xh(STREAM * fd,XL_SEXP * s,int flags)
+_print_field_xh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 XL_SYM_FIELD * sf;
 	if ( (flags & PFM_FORMAT) == PF_HTML &&
@@ -209,19 +242,19 @@ XL_SYM_FIELD * sf;
 			switch ( sf->data[0] ) {
 			case '#':
 				if ( sf->data[1] == '-' )
-					s_printf(fd," %ls=\"#%ls\"",
+					_e_printf(fd,ee," %ls=\"#%ls\"",
 						sf->name,
 						&sf->data[2]);
-				else	s_printf(fd," %ls=%ls ",
+				else	_e_printf(fd,ee," %ls=%ls ",
 						sf->name,
 						&sf->data[1]);
 				break;
 			case 0:
-				s_printf(fd," %ls",
+				_e_printf(fd,ee," %ls",
 					sf->name);
 				break;
 			default:
-				s_printf(fd," %ls=\"%ls\"",
+				_e_printf(fd,ee," %ls=\"%ls\"",
 					sf->name,
 					sf->data);
 			}
@@ -229,20 +262,20 @@ XL_SYM_FIELD * sf;
 	}
 	else {
 		for ( sf = s->symbol.field ; sf ; sf = sf->next )
-			s_printf(fd," %ls=\"%ls\"",
+			_e_printf(fd,ee," %ls=\"%ls\"",
 				sf->name,
 				sf->data);
 	}
 }
 
 void
-__print_field_l(STREAM * fd,L_CHAR * str)
+__print_field_l(STREAM * fd,ERR_ENV * ee,L_CHAR * str)
 {
 L_CHAR * p1;
 int len;
 L_CHAR * buf;
 	p1 = str;
-	s_printf(fd,"\"");
+	_e_printf(fd,ee,"\"");
 	buf = d_alloc(10);
 	for ( ; ; ) {
 		for ( len = 0 ; p1[len] != '\n' &&
@@ -259,26 +292,26 @@ L_CHAR * buf;
 		buf[len] = 0;
 		switch ( p1[len] ) {
 		case 0:
-			s_printf(fd,"%ls\"",buf);
+			_e_printf(fd,ee,"%ls\"",buf);
 			break;
 		case '\n':
 		case '\r':
-			s_printf(fd,"%ls\\n",buf);
+			_e_printf(fd,ee,"%ls\\n",buf);
 			break;
 		case '\t':
-			s_printf(fd,"%ls\\t",buf);
+			_e_printf(fd,ee,"%ls\\t",buf);
 			break;
 		case '\\':
-			s_printf(fd,"%ls\\\\",buf);
+			_e_printf(fd,ee,"%ls\\\\",buf);
 			break;
 		case '"':
-			s_printf(fd,"%ls\\\"",buf);
+			_e_printf(fd,ee,"%ls\\\"",buf);
 			break;
 		case '&':
-			s_printf(fd,"%ls&amp;",buf);
+			_e_printf(fd,ee,"%ls&amp;",buf);
 			break;
 		case '<':
-			s_printf(fd,"%ls&lt;",buf);
+			_e_printf(fd,ee,"%ls&lt;",buf);
 			break;
 		}
 		if ( p1[len] == 0 )
@@ -289,28 +322,28 @@ L_CHAR * buf;
 }
 
 void
-_print_field_l(STREAM * fd,XL_SEXP * s,int flags)
+_print_field_l(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 XL_SYM_FIELD * sf;
 	for ( sf = s->symbol.field ; sf ; sf = sf->next ) {
-		s_printf(fd," %ls=", sf->name);
-		__print_field_l(fd,sf->data);
+		_e_printf(fd,ee," %ls=", sf->name);
+		__print_field_l(fd,ee,sf->data);
 	}
 }
 
 void
-_print_pair_xh(STREAM * fd,XL_SEXP * s,int flags,INDENT ind)
+_print_pair_xh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags,INDENT ind)
 {
 XL_SEXP * s1;
 XL_SEXP * sym;
 XL_SEXP * r;
 
 	if ( get_type(sym=car(s)) != XLT_SYMBOL ) {
-		_print_pair_l(fd,s,flags,ind);
+		_print_pair_l(fd,ee,s,flags,ind);
 		return;
 	}
 	if ( check_valid_tag(sym->symbol.data) ) {
-		_print_pair_l(fd,s,flags,ind);
+		_print_pair_l(fd,ee,s,flags,ind);
 		return;
 	}
 	if ( get_type(s1=cdr(s)) != XLT_NULL ) {
@@ -323,34 +356,34 @@ XL_SEXP * r;
 */
 		ind.flags = 0;
 		ind.ind ++;
-		s_printf(fd,"<%ls",sym->symbol.data);
-		_print_field_xh(fd,sym,flags&(~PF_LISP_MODE));
+		_e_printf(fd,ee,"<%ls",sym->symbol.data);
+		_print_field_xh(fd,ee,sym,flags&(~PF_LISP_MODE));
 		if ( flags&PF_INDENT )
-			s_printf(fd,">\n");
-		else	s_printf(fd,">");
+			_e_printf(fd,ee,">\n");
+		else	_e_printf(fd,ee,">");
 		s1 = cdr(s);
 		for ( ; get_type(s1) == XLT_PAIR ; s1 = cdr(s1) ) {
 			if ( flags&PF_INDENT )
-				print_ind(fd,ind);
-			_print_sexp(fd,car(s1),flags&(~PF_LISP_MODE),ind);
+				print_ind(fd,ee,ind);
+			_print_sexp(fd,ee,car(s1),flags&(~PF_LISP_MODE),ind);
 			if ( flags&PF_INDENT )
-				s_printf(fd,"\n");
-			else	s_printf(fd," ");
+				_e_printf(fd,ee,"\n");
+			else	_e_printf(fd,ee," ");
 		}
 		ind.ind --;
 		if ( flags&PF_INDENT )
-			print_ind(fd,ind);
-		s_printf(fd,"</%ls>",sym->symbol.data);
+			print_ind(fd,ee,ind);
+		_e_printf(fd,ee,"</%ls>",sym->symbol.data);
 	}
 	else {
-		s_printf(fd,"<%ls",sym->symbol.data);
-		_print_field_xh(fd,sym,flags&(~PF_LISP_MODE));
+		_e_printf(fd,ee,"<%ls",sym->symbol.data);
+		_print_field_xh(fd,ee,sym,flags&(~PF_LISP_MODE));
 		if ( (flags&PFM_FORMAT) == PF_XML ) {
 			if ( sym->symbol.data[0] == '?' )
-				s_printf(fd,"?>");
-			else	s_printf(fd,"/>");
+				_e_printf(fd,ee,"?>");
+			else	_e_printf(fd,ee,"/>");
 		}
-		else	s_printf(fd,">");
+		else	_e_printf(fd,ee,">");
 	}
 }
 
@@ -360,17 +393,17 @@ pt_t pt_pair = {
 };
 
 void
-_print_symbol_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_symbol_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 	if ( !(flags&PF_LISP_MODE) )
-		s_printf(fd,"^");
+		_e_printf(fd,ee,"^");
 	if ( s->symbol.field ) {
-		s_printf(fd,"[%ls",s->symbol.data);
-		_print_field_l(fd,s,flags);
-		s_printf(fd,"]");
+		_e_printf(fd,ee,"[%ls",s->symbol.data);
+		_print_field_l(fd,ee,s,flags);
+		_e_printf(fd,ee,"]");
 	}
 	else {
-		s_printf(fd,"%ls",s->symbol.data);
+		_e_printf(fd,ee,"%ls",s->symbol.data);
 	}
 }
 
@@ -380,19 +413,19 @@ pt_t pt_symbol = {
 };
 
 void
-_print_string_l(STREAM * fd,XL_SEXP * s,int flags)
+_print_string_l(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 L_CHAR * p1;
 int len;
 L_CHAR * buf;
 	if ( flags&PF_TEXT ) {
-		s_printf(fd,"%ls",s->string.data);
+		_e_printf(fd,ee,"%ls",s->string.data);
 		return;
 	}
 	p1 = s->string.data;
 	if ( !(flags&PF_LISP_MODE) )
-		s_printf(fd,"^\"");
-	else	s_printf(fd,"\"");
+		_e_printf(fd,ee,"^\"");
+	else	_e_printf(fd,ee,"\"");
 	buf = d_alloc(10);
 	for ( ; ; ) {
 		for ( len = 0 ; p1[len] != '\n' &&
@@ -409,26 +442,26 @@ L_CHAR * buf;
 		buf[len] = 0;
 		switch ( p1[len] ) {
 		case 0:
-			s_printf(fd,"%ls\"",buf);
+			_e_printf(fd,ee,"%ls\"",buf);
 			break;
 		case '\n':
 		case '\r':
-			s_printf(fd,"%ls\\n",buf);
+			_e_printf(fd,ee,"%ls\\n",buf);
 			break;
 		case '\t':
-			s_printf(fd,"%ls\\t",buf);
+			_e_printf(fd,ee,"%ls\\t",buf);
 			break;
 		case '\\':
-			s_printf(fd,"%ls\\\\",buf);
+			_e_printf(fd,ee,"%ls\\\\",buf);
 			break;
 		case '"':
-			s_printf(fd,"%ls\\\"",buf);
+			_e_printf(fd,ee,"%ls\\\"",buf);
 			break;
 		case '&':
-			s_printf(fd,"%ls&amp;",buf);
+			_e_printf(fd,ee,"%ls&amp;",buf);
 			break;
 		case '<':
-			s_printf(fd,"%ls&lt;",buf);
+			_e_printf(fd,ee,"%ls&lt;",buf);
 			break;
 		}
 		if ( p1[len] == 0 )
@@ -439,37 +472,37 @@ L_CHAR * buf;
 }
 
 void
-_print_string_xh(STREAM * fd,XL_SEXP * s,int flags)
+_print_string_xh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 L_CHAR * p1;
 int len;
 L_CHAR * buf;
 	if ( flags&PF_TEXT ) {
-		s_printf(fd,"%ls",s->string.data);
+		_e_printf(fd,ee,"%ls",s->string.data);
 		return;
 	}
 	if ( flags&PF_LISP_MODE ) {
-		_print_string_l(fd,s,flags);
+		_print_string_l(fd,ee,s,flags);
 		return;
 	}
 	if ( s->string.data[0] == '^' ) {
-		_print_string_l(fd,s,flags);
+		_print_string_l(fd,ee,s,flags);
 		return;
 	}
 	if ( s->string.data[0] == 0 ) {
-		_print_string_l(fd,s,flags);
+		_print_string_l(fd,ee,s,flags);
 		return;
 	}
 	if ( '0' <= s->string.data[0] &&
 			s->string.data[0] <= '9' ) {
-		_print_string_l(fd,s,flags);
+		_print_string_l(fd,ee,s,flags);
 		return;
 	}
 	if ( s->string.data[0] == '-' ||
 			s->string.data[0] == '+' ) {
 		if ( '0' <= s->string.data[1] &&
 				s->string.data[1] <= '9' ) {
-			_print_string_l(fd,s,flags);
+			_print_string_l(fd,ee,s,flags);
 			return;
 		}
 	}
@@ -488,21 +521,21 @@ L_CHAR * buf;
 		memcpy(buf,p1,len*sizeof(L_CHAR));
 		buf[len] = 0;
 		if ( 0 < p1[len] && p1[len] <= ' ' ) {
-			s_printf(fd,"%ls&#%i;",buf,
+			_e_printf(fd,ee,"%ls&#%i;",buf,
 				p1[len]);
 		}
 		else switch ( p1[len] ) {
 		case 0:
-			s_printf(fd,"%ls",buf);
+			_e_printf(fd,ee,"%ls",buf);
 			break;
 		case '&':
-			s_printf(fd,"%ls&amp;",buf);
+			_e_printf(fd,ee,"%ls&amp;",buf);
 			break;
 		case '<':
-			s_printf(fd,"%ls&lt;",buf);
+			_e_printf(fd,ee,"%ls&lt;",buf);
 			break;
 		default:
-			s_printf(fd,"%ls&cx%x;",buf,p1[len]);
+			_e_printf(fd,ee,"%ls&cx%x;",buf,p1[len]);
 		}
 		if ( p1[len] == 0 )
 			break;
@@ -517,11 +550,11 @@ pt_t pt_string = {
 };
 
 void
-_print_integer_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_integer_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
-	s_printf(fd,"%i",s->integer.data);
+	_e_printf(fd,ee,"%i",s->integer.data);
 	if ( s->integer.unit )
-		s_printf(fd,"%ls",s->integer.unit);
+		_e_printf(fd,ee,"%ls",s->integer.unit);
 }
 
 pt_t pt_integer = {
@@ -530,12 +563,12 @@ pt_t pt_integer = {
 };
 
 void
-_print_floating_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_floating_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
-	s_printf(fd,"%lf",s->floating.data);
+	_e_printf(fd,ee,"%lf",s->floating.data);
 	if ( s->floating.unit )
-		s_printf(fd,"%ls ",s->floating.unit);
-	else  	s_printf(fd," ");
+		_e_printf(fd,ee,"%ls ",s->floating.unit);
+	else  	_e_printf(fd,ee," ");
 }
 
 pt_t pt_floating = {
@@ -544,17 +577,17 @@ pt_t pt_floating = {
 };
 
 void
-_print_function_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_function_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 	switch ( s->func.type ) {
 	case FT_PRIM:
-		s_printf(fd,"[prim:0x%x]",s->func.prim);
+		_e_printf(fd,ee,"[prim:0x%x]",s->func.prim);
 		break;
 	case FT_LAMBDA:
-		s_printf(fd,"[lambda:0x%x]",s->func.l_body);
+		_e_printf(fd,ee,"[lambda:0x%x]",s->func.l_body);
 		break;
 	default:
-		s_printf(fd,"[%i:]",s->func.type);
+		_e_printf(fd,ee,"[%i:]",s->func.type);
 		break;
 	}
 }
@@ -565,14 +598,14 @@ pt_t pt_function = {
 };
 
 void
-_print_raw_lxh(STREAM * fd,XL_SEXP * s,int flags)
+_print_raw_lxh(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags)
 {
 int er;
 char * pp;
 int size,ss;
 	if ( !(flags&PF_LISP_MODE) )
-		s_printf(fd,"^");
-	s_printf(fd,"#%x#",s->raw.size);
+		_e_printf(fd,ee,"^");
+	_e_printf(fd,ee,"#%x#",s->raw.size);
 	if ( !(flags&PF_RAW_DISABLE) ) {
 		pp = s->raw.data;
 		size = s->raw.size;
@@ -587,13 +620,15 @@ int size,ss;
 						ss = 1;
 					goto retry;
 				}
+				ee->er = errno;
+				longjmp(ee->env,-1);
 				break;
 			}
 			size -= er;
 			pp += er;
 		}
 	}
-	else	s_printf(fd,"]");
+	else	_e_printf(fd,ee,"]");
 	if ( flags&PF_RAW_CLEAR ) {
 		if ( s->raw.data )
 			mfree(s->raw.data);
@@ -647,30 +682,44 @@ pt_t * tbl;
 }
 
 void
-_print_sexp(STREAM * fd,XL_SEXP * s,int flags,INDENT ind)
+_print_sexp(STREAM * fd,ERR_ENV * ee,XL_SEXP * s,int flags,INDENT ind)
 {
 pt_t * tbl;
 	tbl = _get_print_table(s);
 	if ( (flags&PFM_FORMAT) == PF_LISP )
-		((*tbl)[0])(fd,s,flags,ind);
-	else	((*tbl)[1])(fd,s,flags,ind);
+		((*tbl)[0])(fd,ee,s,flags,ind);
+	else	((*tbl)[1])(fd,ee,s,flags,ind);
 }
 
-void
-print_sexp(STREAM * fd,XL_SEXP * s,int flags)
+int
+e_print_sexp(STREAM * fd,int * erp,XL_SEXP * s,int flags)
 {
 pt_t * tbl;
 INDENT ind;
+ERR_ENV ee;
+
+	ee.er = 0;
+	if ( setjmp(ee.env) ) {
+		*erp = ee.er;
+		return -1;
+	}
 
 	ind.flags = 0;
 	ind.ind = 0;
 	if ( flags&PF_MULTI_ROOT ) {
 		for ( ; get_type(s) != XLT_NULL ; s = cdr(s) ) {
-			_print_sexp(fd,car(s),flags,ind);
-			s_printf(fd,"\n");
+			_print_sexp(fd,&ee,car(s),flags,ind);
+			_e_printf(fd,&ee,"\n");
 		}
 	}
 	else {
-		_print_sexp(fd,s,flags,ind);
+		_print_sexp(fd,&ee,s,flags,ind);
 	}
+	return 0;
 }
+
+void
+print_sexp(STREAM * fd,XL_SEXP * s,int flags)
+{
+	e_print_sexp(fd,0,s,flags);
+}
\ No newline at end of file
