@@ -17,6 +17,7 @@
 
 
 #include	"task.h"
+#include	"memory_routine.h"
 
 extern SEM xli_lock;
 extern SEM fifo_lock;
@@ -52,10 +53,25 @@ gc_xl_unlock()
 void
 gc_lib()
 {
+
+	GC_POSITION;
+
 	gc_gblisp();
+
+	GC_POSITION;
+
 	gc_interpreter();
+
+	GC_POSITION;
+
 	gc_gc_stack();
+
+	GC_POSITION;
+
 	gc_unit();
+
+	GC_POSITION;
+
 	gc_queue();
 }
 
