@@ -63,10 +63,36 @@ int interpreter_que_task;
 
 L_CHAR * lock_function;
 LOCK_FUNCTION_TYPE * lf_list;
+int dque_lock_cnt;
 
 int (* agent_open_xli)();
 int (* agent_close_xli)();
 
+
+int default_silent_timeout_table[] = {
+	-1,
+	-1,	/* XLA_STDIO */
+	-1,	/* XLA_FILE */
+	-1,	/* XLA_IPC */
+	-1,	/* XLA_ACCEPT */
+	-1, 	/* XLA_CONNECT */
+	-1,	/* XLA_DESCRIPTER */
+	-1,	/* XLA_SELF */
+	-1	/* XLA_PIPE */
+};
+
+int default_connection_timeout_table[] = {
+	-1,
+	-1,	/* XLA_STDIO */
+	-1,	/* XLA_FILE */
+	-1,	/* XLA_IPC */
+	-1,	/* XLA_ACCEPT */
+	2*3600,	/* XLA_CONNECT */
+	-1,	/* XLA_DESCRIPTER */
+	-1,	/* XLA_SELF */
+	-1	/* XLA_PIPE */
+};
+
 void
 test_thread()
 {
@@ -657,7 +683,6 @@ XL_INTERPRETER * xli;
 	lock_task(xli_lock);
 	xli = _get_my_xli();
 	if ( xli->mode != XIM_RUN ) {
-ss_printf("xli_break_check 1\n");
 		ret = get_error(
 			s->h.file,
 			s->h.line,
@@ -666,7 +691,6 @@ ss_printf("xli_break_check 1\n");
 			0);
 	}
 	if ( xli->flags & XIF_BREAK ) {
-ss_printf("xli_break_check 2\n");
 		ret = get_error(
 			s->h.file,
 			s->h.line,
@@ -863,7 +887,9 @@ setup_xl_interpreter(XL_INTERPRETER * xli)
 {
 unsigned int id,key;
 CODE_METHOD * cm1;
+int er;
 
+	er = 0;
 
 	if ( xli->a_type != XLA_ACCEPT ) {
 		xli->fifo_head = xli->fifo_tail = 0;
@@ -874,11 +900,13 @@ CODE_METHOD * cm1;
 		xli->silent_cnt = -1;
 		xli->connection_cnt = 0;
 	}
-	switch ( xli->a_type ) {
-	case XLA_CONNECT:
-		if ( xli->connection_timeout == -1 )
-			xli->connection_timeout = 2*3600;
-		break;
+	if ( xli->silent_timeout == -1 ) {
+		xli->silent_timeout =
+			 default_silent_timeout_table[xli->a_type];
+	}
+	if ( xli->connection_timeout == -1 ) {
+		xli->connection_timeout = 
+			default_connection_timeout_table[xli->a_type];
 	}
 
 
@@ -927,7 +955,7 @@ CODE_METHOD * cm1;
 		(*agent_open_xli)(xli);
 	switch ( xli->a_type ) {
 	case XLA_CONNECT:
-		s_printf(xli->out,"\n");
+		e_printf(xli->out,&er,"\n");
 		s_flush(xli->out);
 
 		if ( !(xli->flags&XIF_PERMISSION) )
@@ -962,7 +990,7 @@ CODE_METHOD * cm1;
 		if ( id >= 0 ) {
 			cm1 = s_get_cm(xli->out);
 			if ( cm1 )
-				s_printf(xli->out,"&cx%x;\n",cm1->lccode);
+				e_printf(xli->out,&er,"&cx%x;\n",cm1->lccode);
 			xli->flags &= ~XIF_CODE_SYNC;
 		}
 
@@ -977,15 +1005,67 @@ CODE_METHOD * cm1;
 		break;
 	}
 
+	if ( er ) {
+		s_close(xli->out);
+		xli->out = 0;
+		id = XLIE_CANNOT_OPEN_CONNECT;
+	}
+
 	return id;
 }
 
+void
+_dque_lock(XL_INTERPRETER * xli)
+{
+	if ( xli ) {
+		for ( ; dque_lock_cnt < 0 ; ) {
+			sleep_task((int)&dque_lock_cnt,xli_lock);
+			lock_task(xli_lock);
+		}
+		dque_lock_cnt ++;
+		for ( ; (xli->flags & XIF_DQUE_LOCK) ; ) {
+			sleep_task((int)xli,xli_lock);
+			lock_task(xli_lock);
+		}
+		xli->flags |= XIF_DQUE_LOCK;
+	}
+	else {
+		for ( ; dque_lock_cnt > 0 ; ) {
+			sleep_task((int)&dque_lock_cnt,xli_lock);
+			lock_task(xli_lock);
+		}
+		dque_lock_cnt --;
+	}
+}
+
+void
+_dque_unlock(XL_INTERPRETER * xli)
+{
+	if ( xli ) {
+		dque_lock_cnt --;
+		xli->flags &= ~XIF_DQUE_LOCK;
+		wakeup_task((int)xli);
+		wakeup_task((int)&dque_lock_cnt);
+		wakeup_task((int)delay_thread);
+	}
+	else {
+		dque_lock_cnt ++;
+		wakeup_task((int)&dque_lock_cnt);
+	}
+}
+
 void
 insert_delay_que(XL_INTERPRETER * xli,int line,XL_SEXP * s)
 {
 DELAY_QUE_T  * d;
 
 	lock_task(xli_lock);
+	_dque_lock(xli);
+	if ( xli->flags & XIF_DQUE_CLOSE ) {
+		_dque_unlock(xli);
+		unlock_task(xli_lock,"insert_delay_que");
+		return;
+	}
 	xli->delay_que_cnt ++;
 	unlock_task(xli_lock,"insert_delay_que");
 	d = new_queue_node(sizeof(*d));
@@ -994,6 +1074,9 @@ DELAY_QUE_T  * d;
 	d->xli = xli;
 	d->h.key = 0;
 	insert_queue(&interpreter_que,d,0);
+	lock_task(xli_lock);
+	_dque_unlock(xli);
+	unlock_task(xli_lock,"insert_delay_que");
 }
 
 
@@ -1118,6 +1201,7 @@ _close_interpreter(XL_INTERPRETER * xli)
 {
 XL_RESULT * resp;
 int id;
+int er;
 
 	if( xli->mode == XIM_CLOSE )
 		return -1;
@@ -1137,10 +1221,18 @@ int id;
 			resp = resp->my_xli_result_next ) {
 		if ( resp->out_interpreter == 0 )
 			continue;
-		s_printf(resp->out_interpreter->out,
+
+		er = 0;
+		e_printf(resp->out_interpreter->out,
+			&er,
 			"<Cancel/>\n");
 		s_flush(resp->out_interpreter->out);
 		wakeup_task((int)resp);
+
+		if ( er ) {
+			s_close(resp->out_interpreter->out);
+			resp->out_interpreter->out = 0;
+		}
 	}
 
 	s_close(xli->inp);
@@ -1250,13 +1342,13 @@ CODE_METHOD * cm1;
 	setup_xl_interpreter(xli2);
 	gc_pop(0,0);
 
-	if ( xli2->out ) {
-		cm1 = s_get_cm(xli2->out);
+	if ( pip ) {
+		cm1 = s_get_cm(pip);
 		if ( cm1 )
-			s_printf(xli2->out,
+			s_printf(pip,
 			"&cx%x;\n<Permission>Allow</Permission>\n",
 				cm1->lccode);
-		s_flush(xli2->out);
+		s_flush(pip);
 		xli2->flags &= ~XIF_CODE_SYNC;
 	}
 }
@@ -1468,7 +1560,9 @@ __reply(STREAM ** stp,XL_INTERPRETER * xli,int line,XL_SEXP * ret)
 int sc;
 CODE_METHOD * cm1, * cm2, * cm3;
 L_CHAR code;
+int er;
 
+	er = 0;
 	cm1 = 0;
 	if ( !(xli->flags & XIF_FIX_CODE) ) {
 		code = get_proper_code(ret,10);
@@ -1488,9 +1582,9 @@ L_CHAR code;
 				cm2 = 0;
 			}
 			else {
-				s_printf(*stp,"&cx%x;",cm1->lccode);
+				e_printf(*stp,&er,"&cx%x;",cm1->lccode);
 				s_set_cm(*stp,cm1);
-				s_printf(*stp,"\n");
+				e_printf(*stp,&er,"\n");
 				lock_task(xli_lock);
 				xli->flags &= ~XIF_CODE_SYNC;
 				unlock_task(xli_lock,"__reply");
@@ -1508,31 +1602,37 @@ L_CHAR code;
 		lock_task(xli_lock);
 		cm3 = s_get_cm(xli->out);
 		if ( cm3 )
-			s_printf(*stp,"&cx%x;",cm3->lccode);
+			e_printf(*stp,&er,"&cx%x;",cm3->lccode);
 		xli->flags &= ~XIF_CODE_SYNC;
 		unlock_task(xli_lock,"__reply");
 	}
 
 	if ( xli->result_flag )
-		s_printf(*stp,"<Result> %i ",line);
+		e_printf(*stp,&er,"<Result> %i ",line);
 
 	if ( *stp )
-		print_sexp(*stp,ret,xli->ps_flags&(~PF_MULTI_ROOT));
+		e_print_sexp(*stp,&er,ret,xli->ps_flags&(~PF_MULTI_ROOT));
 
 	if ( xli->result_flag )
-		s_printf(*stp,"</Result>\n");
-	else	s_printf(*stp,"\n");
+		e_printf(*stp,&er,"</Result>\n");
+	else	e_printf(*stp,&er,"\n");
 	s_flush(*stp);
 
 	if ( cm2 ) {
-		s_printf(*stp,"&cx%x;",cm2->lccode);
+		e_printf(*stp,&er,"&cx%x;",cm2->lccode);
 		s_set_cm(*stp,cm2);
-		s_printf(*stp,"\n");
+		e_printf(*stp,&er,"\n");
 	}
 
 	xli->silent_cnt = sc;
 
+	if ( er ) {
+		s_close(*stp);
+		*stp = 0;
+	}
+
 	unlock_xli_out(xli);
+
 }
 
 void
@@ -1657,16 +1757,27 @@ void
 _cancel(XL_INTERPRETER * xli)
 {
 XL_RESULT * resp;
+int er;
+STREAM * st;
+	er = 0;
 	lock_task(xli_lock);
 	xli->flags |= XIF_CANCEL;
+
 	for ( resp = xli->result_list ; resp ;
 			resp = resp->my_xli_result_next ) {
 		if ( resp->out_interpreter == 0 )
 			continue;
-		s_printf(resp->out_interpreter->out,
+
+		e_printf(resp->out_interpreter->out,
+			&er,
 			"<Cancel/>\n");
 		s_flush(resp->out_interpreter->out);
 		wakeup_task((int)resp);
+
+		if ( er ) {
+			s_close(resp->out_interpreter->out);
+			resp->out_interpreter->out = 0;
+		}
 	}
 	unlock_task(xli_lock,"_cancel");
 }
@@ -2040,29 +2151,19 @@ int line;
 
 		if ( cmd_log ) {
 			log_printf(LOG_MESSAGE,
+				LOG_LAYER_XL,0,
 				"%i.%i.%i.%i executes",
 				(xli->connect_ip>>24)&0x0ff,
 				(xli->connect_ip>>16)&0x0ff,
 				(xli->connect_ip>>8)&0x0ff,
 				xli->connect_ip&0x0ff);
-			log_print_sexp(LOG_MESSAGE,"cmd",r,0);
+			log_print_sexp(LOG_MESSAGE,LOG_LAYER_XL,0,"cmd",r,0);
 		}
-/*
-fflush(stdout);
-ss_printf("(%i)>>",get_xltime());
-print_sexp(s_stdout,r,PF_RAW_DISABLE);
-ss_printf("\n");
-*/
 
 
 		ret = eval(xli->env,r);
 
 
-/*
-ss_printf("(%i)>>RETURN\n",get_xltime());
-print_sexp(s_stdout,ret,PF_RAW_DISABLE);
-ss_printf("\n");
-*/
 
 		gc_pop(ret,gc_gb_sexp); /* section 1 level 2 */
 
@@ -2102,47 +2203,27 @@ XL_SEXP * s, * r, * ret, * cmd;
 XL_INTERPRETER * xli;
 DELAY_QUE_T * n;
 
-void * stt;
-void * stt2;
-GC_STACK * sttt;
-
-stt2 = get_gc_check_point();
-if ( stt2 ) {
-sttt = stt2;
-fprintf(stderr,"STACK = %x\n",sttt);
-if ( sttt->msg )
-fprintf(stderr,"MSG = %s\n",sttt->msg);
-er_panic("_relay_thread");
-}
 
 set_cpu_thr_type(12);
 set_cpu_msg(0);
 
-stt = get_gc_check_point();
-if ( stt ) {
-sttt = stt;
-fprintf(stderr,"STACK = %x\n",sttt);
-if ( sttt->msg )
-fprintf(stderr,"MSG = %s\n",sttt->msg);
-er_panic("_relay_thread");
-}
-
 
 	xli = (XL_INTERPRETER*)GET_TKEY(k);
 	xli->relay_thread = _new_xli_thread(xli);
 	relay_loop(xli,XFF_BLOCK);
 
 
-set_cpu_msg(11);
-
 	if ( (xli->mode == XIM_RELAY_WAIT ||
 			xli->mode == XIM_QUEUE_WAIT)
 			&& agent_close_xli )
 		(*agent_close_xli)(xli);
 
-set_cpu_msg(12);
+	lock_task(xli_lock);
+	_dque_lock(xli);
+	xli->flags |= XIF_DQUE_CLOSE;
+	unlock_task(xli_lock,"insert_delay_que");
 
-	for ( ; ; ) {
+	for ( ; xli->delay_que_cnt ; ) {
 		n = delete_queue(&interpreter_que,delay_t_check_xli,xli,0);
 		lock_task(xli_lock);
 		xli->delay_que_cnt --;
@@ -2151,17 +2232,13 @@ set_cpu_msg(12);
 			break;
 		d_f_ree(n);
 	}
-
 	lock_task(xli_lock);
-
-set_cpu_msg(13);
+	_dque_unlock(xli);
 
 	close_fifo(xli);
 	switch ( xli->mode ) {
 	case XIM_RUN:
 
-set_cpu_msg(14);
-
 		s_close(xli->inp);
 		if ( xli->inp == xli->out )
 			xli->out = 0;
@@ -2170,13 +2247,9 @@ set_cpu_msg(14);
 		xli->inp = 0;
 		xli->mode = XIM_FLONT_WAIT;
 
-set_cpu_msg(15);
-
 		break;
 	case XIM_QUEUE_WAIT:
 
-set_cpu_msg(16);
-
 		for ( ; xli->mode == XIM_QUEUE_WAIT ; ) {
 			sleep_task((int)&xli->mode,xli_lock);
 			lock_task(xli_lock);
@@ -2184,31 +2257,23 @@ set_cpu_msg(16);
 		if ( xli->mode != XIM_RELAY_WAIT )
 			er_panic("_Relay_thread(1)");
 
-set_cpu_msg(17);
-
 	case XIM_RELAY_WAIT:
 
-set_cpu_msg(18);
 
 		_close_xl_interpreter(xli);
 
-set_cpu_msg(19);
-
 
 		break;
 	default:
 		er_panic("relay_thread");
 	}
 
-set_cpu_msg(20);
-
 
 	_free_xli_thread();
 	unlock_task(xli_lock,"relay_thread");
 
-compare_gc_check_point("relay_check",stt);
 
-	gc_check_empty("relay",stt);
+	gc_check_empty("relay",0);
 
 }
 
@@ -2239,57 +2304,56 @@ XL_INTERPRETER * xli;
 	xli->a_type = XLA_SELF;
 	setup_i(xli);
 
-set_t_msg(1);
 
-	w.flags = CDF_ALL&~(CDF_WAIT);
 	for ( ; ; ) {
-set_t_msg(2);
 		lock_task(xli_lock);
-set_t_msg(3);
 		if ( interpreter_que_task >= 1 ) {
 			unlock_task(xli_lock,"delay_thread");
-set_t_msg(4);
 			break;
 		}
 		interpreter_que_task ++;
-set_t_msg(5);
 		unlock_task(xli_lock,"delay_thread");
 
 		gc_push(0,0,"delay_thread");
-set_t_msg(6);
 
-		n = delete_queue(&interpreter_que,delay_t_check,&w,1);
-set_t_msg(7);
+	loop:
+
+		lock_task(xli_lock);
+		_dque_lock(0);
+		unlock_task(xli_lock,"delay_thread");
+
+		w.flags = CDF_ALL&~(CDF_WAIT);
+		n = delete_queue(&interpreter_que,delay_t_check,&w,0);
+		if ( n == 0 ) {
+			lock_task(xli_lock);
+			_dque_unlock(0);
+
+			new_timeout((int)delay_thread,2);
+			sleep_task((int)delay_thread,xli_lock);
+			del_timeout((int)delay_thread);
+			goto loop;
+		}
 
 		lock_task(xli_lock);
 		interpreter_que_task --;
 		unlock_task(xli_lock,"delay_thread");
-set_t_msg(8);
 
 		if ( w.result == CDT_WAIT_ERR ) {
-set_t_msg(9);
 			create_task(delay_thread,0,PRI_XL_RELAY);
 			get_type(n->ret);
 		}
-set_t_msg(10);
 		_reply(n->xli,n->line,n->ret);
-set_t_msg(11);
 		lock_task(xli_lock);
-set_t_msg(12);
 		n->xli->delay_que_cnt --;
 		n->xli->silent_cnt = 0;
-set_t_msg(13);
+		_dque_unlock(0);
 		unlock_task(xli_lock,"delay_thread");
 		d_f_ree(n);
-set_t_msg(14);
 
 		gc_pop(0,0);
-set_t_msg(15);
 	}
-set_t_msg(16);
 
 	close_self_interpreter();
-set_t_msg(17);
 }
 
 void
