@@ -117,13 +117,8 @@ int ret_no;
 				cmp_round(r->seq_to,seq_no) > 0 )
 			goto ok;
 	}
-set_cpu_msg(80000);
 	goto err;
 ok:
-set_cpu_msg(80000+seq_no);
-/*
-ss_printf("res(%i) %x %i\n",getpid(),r,seq_no);
-*/
 
 	r->ret = ret;
 	r->flags |= XRF_RETURN;
@@ -208,8 +203,9 @@ XL_SEXP * r;
 XL_INTERPRETER * my_xli;
 void gc_gb_sexp();
 CODE_METHOD * cm1;
+int er;
 
-
+	er = 0;
 	my_xli = _get_my_xli();
 	if ( my_xli == 0 )
 		er_panic("_remote_query");
@@ -290,20 +286,20 @@ CODE_METHOD * cm1;
 	if ( xli->flags & XIF_CODE_SYNC ) {
 		cm1 = s_get_cm(xli->out);
 		if ( cm1 )
-			s_printf(xli->out,"&cx%x;",cm1->lccode);
+			e_printf(xli->out,&er,"&cx%x;",cm1->lccode);
 		xli->flags &= ~XIF_CODE_SYNC;
 	}
 	if ( cmd_str )
-		s_printf(xli->out,"%ls\n",cmd_str);
+		e_printf(xli->out,&er,"%ls\n",cmd_str);
 	else {
 /*
 ss_printf("seq %i ",resp->seq_from);
 print_sexp(s_stdout,cmd_sexp,0);
 ss_printf("\n");
 */
-		print_sexp(xli->out,cmd_sexp,
+		e_print_sexp(xli->out,&er,cmd_sexp,
 			xli->ps_flags&(~PF_MULTI_ROOT));
-		s_printf(xli->out,"\n");
+		e_printf(xli->out,&er,"\n");
 	}
 
 	resp->seq_to = xli->out->h.cr_cnt;
@@ -317,6 +313,17 @@ ss_printf("\n");
 	resp->my_xli_result_next = my_xli->result_list;
 	my_xli->result_list = resp;
 
+	if ( er ) {
+		s_close(xli->out);
+		xli->out = 0;
+		return get_error(
+			resp->file,
+			resp->line,
+			XLE_PROTO_ACCESS_STREAM,
+			l_string(std_cm,"Remote"),
+			n_get_string("output stream error(last)"));
+	}
+
 	return init_delay_func(&resp->h);
 }
 
@@ -686,7 +693,7 @@ XL_INTERPRETER * xli;
 int iid;
 	gc_push(0,0,"aboat_remote_query");
 	lock_task(xli_lock);
-	if ( get_delay_info(&st,(XL_DELAY**)&resp,s) == CDT_READY ) {
+	if ( get_delay_info(&st,(DELAY_FUNC**)&resp,s) == CDT_READY ) {
 		xli = resp->out_interpreter;
 		if ( xli )
 			iid = xli->id;
