@@ -128,7 +128,7 @@ NPROXY_DIRECT_SERVER *	np_server_list;
 void agent_timeout_task();
 
 void
-init_session()
+init_session_xlp()
 {
 	session_lock = new_lock(LL_SESSION);
 	new_tick(session_tick,CHECK_INTERVAL,0);
@@ -672,12 +672,12 @@ CON_SERVER * s;
 }
 
 int
-open_session(int type)
+open_session_xlp(int type)
 {
 CON_SESSION * ret;
-	lock_task(session_lock);
+//	lock_task(session_lock);
 	ret = _new_con_session(type);
-	unlock_task(session_lock,"open_session");
+//	unlock_task(session_lock,"open_session");
 	return ret->id;
 }
 
@@ -714,7 +714,7 @@ CON_SESSION ** sp, * s;
 }
 
 void
-close_session(int id)
+close_session_xlp(int id)
 {
 	lock_task(session_lock);
 	_close_session(id);
@@ -1543,7 +1543,7 @@ NPROXY_DIRECT_SERVER * ret;
 
 
 XL_SEXP *
-remote_session(
+remote_session_xlp(
 	XLISP_ENV * env,
 	int id,
 	URL * u,
@@ -1559,7 +1559,9 @@ NPROXY_DIRECT_SERVER * is;
 XL_SEXP * send_query, * ret;
 char buf[20];
 L_CHAR * a_agent,* a_login_mode,* a_center_cmd;
-	if ( _a_agent )
+	if ( u->agent )
+		a_agent = ll_copy_str(u->agent);
+	else if ( _a_agent )
 		a_agent = ll_copy_str(_a_agent);
 	else	a_agent = 0;
 	if ( _a_login_mode )
@@ -1568,6 +1570,7 @@ L_CHAR * a_agent,* a_login_mode,* a_center_cmd;
 	if ( _a_center_cmd )
 		a_center_cmd = ll_copy_str(_a_center_cmd);
 	else	a_center_cmd = 0;
+
 	if ( native_proxy_session == 0 ) {
 		ret = _remote_session(
 				env,
@@ -1653,14 +1656,6 @@ L_CHAR * a_agent,* a_login_mode,* a_center_cmd;
 	send_query = cons(
 		n_get_symbol("RemoteSession"),
 		send_query);
-/*
-ss_printf("SEND ");
-print_sexp(s_stdout,send_query,0);
-ss_printf("\n");
- if ( a_center_cmd && l_strcmp(a_center_cmd,
-		l_string(std_cm,"://")) == 0 )
-er_panic("sopt");
-*/
 
 	ret = _remote_session(
 			env,
@@ -1754,16 +1749,17 @@ extern SEM xli_lock;
 	ret = _check_remote_delay(resp,key);
 
 	if ( ret == CDT_READY ) {
-		if ( resp->ret->h.type == XLT_ERROR ) {
-			switch ( resp->ret->err.code ) {
-			case XLE_PROTO_ACCESS_STREAM:
-			case XLE_PROTO_INV_IID:
-				ret = CDT_WAIT_ERR;
-				break;
-			default:
-				ret = CDT_READY;
+		if ( resp->ret )
+			if ( resp->ret->h.type == XLT_ERROR ) {
+				switch ( resp->ret->err.code ) {
+				case XLE_PROTO_ACCESS_STREAM:
+				case XLE_PROTO_INV_IID:
+					ret = CDT_WAIT_ERR;
+					break;
+				default:
+					ret = CDT_READY;
+				}
 			}
-		}
 	}
 	unlock_task(xli_lock,"remote_delay_func");
 	return ret;
